Package: RCAS
Type: Package
Title: RNA Centric Annotation System
Version: 1.36.0
Date: 2024-03-19
Authors@R: c(person("Bora", "Uyar", email='bora.uyar@mdc-berlin.de', role=c("aut", "cre")),
              person("Dilmurat", "Yusuf", role=c("aut")),
              person("Ricardo", "Wurmus", role=c("aut")),
              person("Altuna", "Akalin", role=c("aut")))
Description: RCAS is an R/Bioconductor package designed as a generic reporting tool for the
    functional analysis of transcriptome-wide regions of interest detected by
    high-throughput experiments. Such transcriptomic regions could be, for instance,
    signal peaks detected by CLIP-Seq analysis for protein-RNA interaction sites,
    RNA modification sites (alias the epitranscriptome), CAGE-tag locations, or any
    other collection of query regions at the level of the transcriptome. RCAS
    produces in-depth annotation summaries and coverage profiles based on the
    distribution of the query regions with respect to transcript features (exons,
    introns, 5'/3' UTR regions, exon-intron boundaries, promoter regions). Moreover,
    RCAS can carry out functional enrichment analyses and discriminative motif
    discovery. 
License: Artistic-2.0
LazyData: TRUE
Depends: R (>= 3.5.0), plotly (>= 4.5.2), DT (>= 0.2), data.table
Imports: GenomicRanges, IRanges, BSgenome, BSgenome.Hsapiens.UCSC.hg19,
        GenomeInfoDb (>= 1.12.0), Biostrings, rtracklayer,
        GenomicFeatures, txdbmaker, rmarkdown (>= 0.9.5), genomation
        (>= 1.5.5), knitr (>= 1.12.3), BiocGenerics, S4Vectors,
        plotrix, pbapply, RSQLite, proxy, pheatmap, ggplot2, cowplot,
        seqLogo, utils, ranger, gprofiler2
RoxygenNote: 7.3.1
Suggests: testthat, covr, BiocManager
SystemRequirements: pandoc (>= 1.12.3)
VignetteBuilder: knitr
biocViews: Software, GeneTarget, MotifAnnotation, MotifDiscovery, GO,
        Transcriptomics, GenomeAnnotation, GeneSetEnrichment, Coverage
git_url: https://git.bioconductor.org/packages/RCAS
git_branch: RELEASE_3_22
git_last_commit: 780ed18
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:59:51 UTC; biocbuild
Author: Bora Uyar [aut, cre],
  Dilmurat Yusuf [aut],
  Ricardo Wurmus [aut],
  Altuna Akalin [aut]
Maintainer: Bora Uyar <bora.uyar@mdc-berlin.de>

# Generated by roxygen2: do not edit by hand

export(calculateCoverageProfile)
export(calculateCoverageProfileFromTxdb)
export(calculateCoverageProfileList)
export(calculateCoverageProfileListFromTxdb)
export(checkSeqDb)
export(createControlRegions)
export(createDB)
export(createOrthologousGeneSetList)
export(deleteSampleDataFromDB)
export(discoverFeatureSpecificMotifs)
export(extractSequences)
export(findDifferentialMotifs)
export(findEnrichedFunctions)
export(generateKmers)
export(getFeatureBoundaryCoverage)
export(getFeatureBoundaryCoverageBin)
export(getFeatureBoundaryCoverageMulti)
export(getIntervalOverlapMatrix)
export(getMotifSummaryTable)
export(getPWM)
export(getTargetedGenesTable)
export(getTxdbFeatures)
export(getTxdbFeaturesFromGRanges)
export(importBed)
export(importBedFiles)
export(importGtf)
export(parseMsigdb)
export(plotFeatureBoundaryCoverage)
export(printMsigdbDataset)
export(queryGff)
export(retrieveOrthologs)
export(runGSEA)
export(runMotifDiscovery)
export(runMotifRG)
export(runReport)
export(runReportMetaAnalysis)
export(runTopGO)
export(summarizeDatabaseContent)
export(summarizeQueryRegions)
export(summarizeQueryRegionsMulti)
import(DT)
import(GenomicRanges)
import(RSQLite)
import(ggplot2)
import(knitr)
import(ranger)
import(rmarkdown)
importFrom(BSgenome,available.genomes)
importFrom(BSgenome.Hsapiens.UCSC.hg19,Hsapiens)
importFrom(BiocGenerics,unlist)
importFrom(Biostrings,getSeq)
importFrom(Biostrings,vmatchPattern)
importFrom(GenomeInfoDb,keepStandardChromosomes)
importFrom(GenomeInfoDb,seqlevelsStyle)
importFrom(GenomicFeatures,cdsBy)
importFrom(GenomicFeatures,exonsBy)
importFrom(GenomicFeatures,fiveUTRsByTranscript)
importFrom(GenomicFeatures,intronsByTranscript)
importFrom(GenomicFeatures,promoters)
importFrom(GenomicFeatures,threeUTRsByTranscript)
importFrom(GenomicFeatures,transcripts)
importFrom(GenomicRanges,GRangesList)
importFrom(IRanges,width)
importFrom(RSQLite,SQLite)
importFrom(RSQLite,dbConnect)
importFrom(RSQLite,dbDisconnect)
importFrom(RSQLite,dbWriteTable)
importFrom(S4Vectors,Reduce)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,subjectHits)
importFrom(cowplot,plot_grid)
importFrom(data.table,as.data.table)
importFrom(data.table,data.table)
importFrom(data.table,setkey)
importFrom(genomation,ScoreMatrix)
importFrom(genomation,ScoreMatrixBin)
importFrom(gprofiler2,gost)
importFrom(parallel,clusterExport)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(pbapply,pblapply)
importFrom(pbapply,pbsapply)
importFrom(pheatmap,pheatmap)
importFrom(plotly,add_ribbons)
importFrom(plotly,add_trace)
importFrom(plotly,ggplotly)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(plotrix,std.error)
importFrom(proxy,dist)
importFrom(rtracklayer,import.bed)
importFrom(rtracklayer,import.gff)
importFrom(seqLogo,seqLogo)
importFrom(stats,fisher.test)
importFrom(txdbmaker,makeTxDbFromGRanges)
importFrom(utils,read.table)

#' createControlRegions
#'
#' Given a GRanges object of query regions, create a background set of peaks
#' that have the same length distribution based on the flanking regions of the
#' peaks.
#'
#' @param queryRegions GRanges object containing coordinates of input query
#'   regions imported by the \code{\link{importBed}} function.
#' @return GRanges object that contains the same number of regions as query
#'   regions
#' @examples
#' data(queryRegions)
#' controlRegions <- createControlRegions(queryRegions = queryRegions)
#'
#' @import GenomicRanges
#' @export
createControlRegions <- function (queryRegions) {

  upFlank <- GenomicRanges::flank(queryRegions, start = TRUE, width=200)
  downFlank <- GenomicRanges::flank(queryRegions, start = FALSE, width=200)

  #randomly select up or downstream flanking regions
  s <- sample(c(0:1), length(queryRegions), replace = TRUE)
  flanks <- upFlank
  flanks[s==1] <- downFlank[s==1]

  df <- data.frame(seqnames = GenomicRanges::seqnames(queryRegions),
                   start    = GenomicRanges::start(queryRegions),
                   end      = GenomicRanges::end(queryRegions),
                   strand   = GenomicRanges::strand(queryRegions),
                   width    = GenomicRanges::width(queryRegions))

  df$flankStart <- GenomicRanges::start(flanks)
  df$flankEnd <- GenomicRanges::end(flanks)
  df$randomFlankStart <- apply(X = df[,6:7], 1,
                               FUN = function(x) sample(x[1]:x[2], 1))
  df$randomFlankEnd <- df$randomFlankStart + df$width - 1

  controlRegions <- df[1] #control intervals
  controlRegions$start <- df$randomFlankStart
  controlRegions$end <- df$randomFlankEnd
  controlRegions$id <- paste(controlRegions$seqnames,
                             controlRegions$start,
                             controlRegions$end,
                             df$strand, sep=':')
  controlRegions$score <- 1
  controlRegions$strand <- df$strand

  #convert to Granges object
  controlRegions <- GenomicRanges::makeGRangesFromDataFrame(controlRegions)

  return(controlRegions)
}

#' extractSequences
#'
#' Given a GRanges object and a genome version (hg19, mm9, ce10 or dm3), this
#' function extracts the DNA sequences for all genomic regions found in an input
#' object.
#'
#' @param queryRegions GRanges object containing coordinates of input query
#'   regions imported by the \code{\link{importBed}} function
#' @param genomeVersion A character string to denote the BS genome library
#'   required to extract sequences. Available options are hg19, mm9, ce10 and
#'   dm3.
#'
#' @return DNAStringSet object will be returned
#'
#' @examples
#'
#' data(queryRegions)
#' sequences <- extractSequences(queryRegions = queryRegions,
#'                              genomeVersion = 'hg19')
#' @importFrom BSgenome.Hsapiens.UCSC.hg19 Hsapiens
#' @importFrom GenomeInfoDb seqlevelsStyle
#' @importFrom Biostrings getSeq
#' @export
extractSequences <- function (queryRegions, genomeVersion) {
  
  seqDb <- checkSeqDb(genomeVersion)
  style <- GenomeInfoDb::seqlevelsStyle(queryRegions)
  GenomeInfoDb::seqlevelsStyle(seqDb) <- style
  sequences <- Biostrings::getSeq(seqDb, queryRegions)
  names(sequences) <- paste('seq', 1:length(sequences), sep='_')

  return (sequences)
}

#' Generate K-mers
#' 
#' Given a list of characters, generates all possible fixed length strings 
#' 
#' @param k The length of the strings to be generated 
#' @param letters A character vector 
#' @return Vector of strings 
#' @examples
#' generateKmers(3, c('A', 'C', 'G'))
#' 
#' @export
generateKmers <- function(k, letters = c("A", "C", "G", "T")) {
  kmer <- c()
  for(i in 1:k){
    kmer <- unlist(lapply(letters, function(x){paste(kmer, x, sep="")}))
  }
  return(kmer)
}

countPattern <- function(seqs, patterns, maxMismatch = 0, nCores = 1) {
  
  cl <- parallel::makeCluster(nCores)
  parallel::clusterExport(cl = cl, varlist = c('seqs', 'patterns', 'maxMismatch'), 
                            envir = environment())
  M <- do.call(rbind, pbapply::pblapply(cl = cl, X = patterns, 
                               FUN = function(x) {
                                 Biostrings::vcountPattern(x, seqs, 
                                                           max.mismatch = maxMismatch)
                               }))
  parallel::stopCluster(cl)
  rownames(M) <- patterns
  return(t(M))
}

#' @importFrom IRanges width
#' @importFrom Biostrings vmatchPattern
extractMatches <- function(seqs, patterns, minMismatch = 0, maxMismatch = 0) {
  matches <- lapply(patterns, function(x) {
    m <- Biostrings::vmatchPattern(pattern = x, subject = seqs, max.mismatch = maxMismatch)
    m <- unlist(m[lengths(m) > 0])
    if(length(m) == 0) {
      return(NULL)
    }
    # remove out of bounds matches
    m <- m[start(m) >= 1]
    m <- m[end(m) <= IRanges::width(seqs[names(m)])]
    m <- split(m, names(m))
    common <- intersect(names(m), names(seqs))
    if(length(common) == 0) {
      return(NULL)
    }
    paste(unlist(Biostrings::extractAt(seqs[common], m[common])))
  })
  names(matches) <- patterns
  return(matches)
}

#' Find Differential Motifs 
#' 
#' @param querySeqs A DNAStringSet object that is the regions of interest. 
#' @param controlSeqs A DNAStrintSet object that serve as the control
#' @param motifWidth A Positive integer (default: 6) for the generated k-mers. Warning: we recommend
#' using values below 10 as the computation gets exponentially difficult as the 
#' motif width is increased. 
#' @param motifN A positive integer (default:1) denoting the maximum number of 
#'   motifs that should be returned by the \code{findDifferentialMotifs} function
#' @param maxMismatch A positive integer (default: 1) - maximum number of
#'   mismatches to allow when searching for k-mer matches in sequences.
#' @param nCores A positive integer (default:1) number of cores used for 
#'   parallel execution. 
#' @import ranger
#' @examples 
#' data(queryRegions)
#' 
#' # get query and control sequences
#' querySeqs <- extractSequences(queryRegions[1:500], 'hg19')
#' controlRegions <- createControlRegions(queryRegions[1:500])
#' controlSeqs <- extractSequences(controlRegions, 'hg19')
#' 
#' #run motif discovery
#' motifResults <- findDifferentialMotifs(querySeqs = querySeqs, 
#'                                        controlSeqs = controlSeqs, 
#'                                        motifWidth = 5,
#'                                        motifN = 1,
#'                                        maxMismatch = 0, 
#'                                        nCores = 1)
#' #summarize motif results
#' getMotifSummaryTable(motifResults)
#' @export
findDifferentialMotifs <- function(querySeqs, 
                                   controlSeqs, 
                                   motifWidth = 6,
                                   motifN = 1, 
                                   nCores = 1, 
                                   maxMismatch = 1) {
  
  kmers <- generateKmers(k = motifWidth)
  
  selected <- seq(1:length(querySeqs))
  if(length(querySeqs) > 1000) {
    selected <- sample(1:length(querySeqs), 1000)
  }
  
  query <- countPattern(querySeqs[selected], kmers, maxMismatch, nCores)
  ctrl <- countPattern(controlSeqs[selected], kmers, maxMismatch, nCores)

  # only keep patterns that are more frequent in the query
  queryHits <- apply(query, 2, function(x) sum(x > 0))
  controlHits <- apply(ctrl, 2, function(x) sum(x > 0))
  
  candidates <- names(which(log2((queryHits + 1) / (controlHits+1)) > 0))
  
  if(length(candidates) == 0) {
    warning("Couldn't find any motifs that occur more often in the query sequences
            compared to the background. Returning NULL. ")
    return(NULL)
  }
  
  query <- query[,candidates]
  ctrl <- ctrl[,candidates]
  
  # train a random forest model 
  df <- as.data.frame(rbind(query[,candidates], ctrl[,candidates]))
  df$label <- as.factor(c(rep("query", nrow(query)), 
                          rep("ctrl", nrow(ctrl))))
  
  fit <- ranger::ranger(label ~ ., df, importance = 'impurity')
  var.imp <- sort(ranger::importance(fit), decreasing = T)
  #get top variables 
  max <- ifelse(motifN < length(candidates), motifN, length(candidates))
  top <- names(var.imp[1:max])

  results <- list("counts_query" = query[,top, drop = F], 
                  "counts_ctrl" = ctrl[,top,drop = F], 
                  "matches_query" = extractMatches(seqs = querySeqs, 
                                                   patterns = top, 
                                                   maxMismatch = maxMismatch), 
                  "matches_ctrl" = extractMatches(seqs = controlSeqs, 
                                                  patterns = top, 
                                                  maxMismatch = maxMismatch))
  return(results)
}

#' run motifRG 
#' 
#' @export
runMotifRG <- function() {
  .Deprecated("runMotifDiscovery")
}

#' runMotifDiscovery
#'
#' This function builds a random forest classifier to find the top most
#' discriminative motifs in the query regions compared to the background. The
#' background sequences are automatically generated based on the query regions.
#' First, k-mers of a fixed length are generated. The query and control sequences
#' are searched for k-mers allowing for mismatches. A random forest model is 
#' trained to find the most discriminative motifs.  
#' 
#' @param queryRegions GRanges object containing coordinates of input query
#'   regions imported by the \code{\link{importBed}} function
#' @param resizeN Integer value (default: 0) to resize query regions if they are
#'   shorter than the value of \code{resize}. Set to 0 to disable resize.
#' @param sampleN A positive integer value. The queryRegions are randomly
#'   downsampled to include intervals as many as \code{sampleN}. The input will
#'   be downsampled only if this value is larger than zero and less than the
#'   total number of input intervals.
#' @param genomeVersion A character string to denote the BS genome library
#'   required to extract sequences. Example: 'hg19'
#' @param motifWidth A Positive integer (default: 6) for the generated k-mers.
#'   Warning: we recommend using values below 10 as the computation gets
#'   exponentially difficult as the motif width is increased.
#' @param motifN A positive integer (default:5) denoting the maximum number of
#'   motifs that should be returned by the \code{findDifferentialMotifs}
#'   function
#' @param maxMismatch A positive integer (default: 1) - maximum number of
#'   mismatches to allow when searching for k-mer matches in sequences.
#' @param nCores A positive integer (default:1) number of cores used for
#'   parallel execution.
#' @return A list of four objects: k-mer count matrices for query and background
#'   and lists of string matches for the top discriminating motifs (motifN).
#'   
#' @examples
#' data(queryRegions)
#' motifResults <- runMotifDiscovery(queryRegions = queryRegions[1:1000],
#'                                   genomeVersion = 'hg19',
#'                                   motifWidth = 6, 
#'                                   resize = 15,
#'                                   motifN = 1,
#'                                   maxMismatch = 1,
#'                                   nCores = 1)
#' @export
runMotifDiscovery <- function (queryRegions, resizeN = 0, motifWidth = 6,
                        sampleN = 0, genomeVersion, maxMismatch = 1,
                        motifN = 5, nCores = 1) {

  if(sampleN > 0 && length(queryRegions) > sampleN) {
    message("Randomly sampling query regions for motif analysis. 
            Downsampling to ",sampleN," regions")
    queryRegions <- sample(queryRegions, sampleN)
  } 
  
  if(resizeN > 0) {
    resizeIntervals <- width(queryRegions) < resizeN
    if(sum(resizeIntervals) > 0){
      message("Found ",sum(resizeIntervals)," query regions shorter than ", 
              resizeN, " bps. Resizing those regions to ",resizeN,' bps')
      queryRegions[resizeIntervals] <- GenomicRanges::resize(
        x = queryRegions[resizeIntervals], 
        width = 15, 
        fix = 'center')
    }
  }

  message('extracting sequences from fasta..')
  querySeqs <- extractSequences(queryRegions, genomeVersion)

  message('extracting background sequences from fasta..')
  controlRegions <- createControlRegions(queryRegions)
  controlSeqs <- extractSequences(controlRegions, genomeVersion)

  message('running motif discovery ... ')
  motifResults <- findDifferentialMotifs(querySeqs = querySeqs, 
                                         controlSeqs = controlSeqs, 
                                         motifWidth = motifWidth, 
                                         motifN = motifN,
                                         maxMismatch = maxMismatch, 
                                         nCores = nCores)
  return(motifResults)
}

#' getMotifSummaryTable
#'
#' Get summary stats for top discovered motifs 
#'
#' @param motifResults Output object of \code{runMotifDiscovery} function
#' @return A data.frame object containing summary statistics about the
#'   discovered motifs
#' @examples
#'
#' data(queryRegions)
#' motifResults <- runMotifDiscovery(queryRegions = queryRegions[1:1000],
#'                                   genomeVersion = 'hg19',
#'                                   resize = 15,
#'                                   motifN = 1,
#'                                   maxMismatch = 1,
#'                                   nCores = 2)
#' motifSummary <- getMotifSummaryTable(motifResults)
#' @importFrom stats fisher.test
#' @export
getMotifSummaryTable <- function(motifResults){

  if(!is.null(motifResults)) {
    df <- data.frame('patterns' = names(motifResults$matches_query),
               'queryHits' = colSums(motifResults$counts_query), # total number of motifs in sequences
               'controlHits' = colSums(motifResults$counts_ctrl), 
               'querySeqs' = colSums(motifResults$counts_query > 0), #number of seqs with motifs 
               'controlSeqs' = colSums(motifResults$counts_ctrl > 0),
               'queryFraction' = round(colSums(motifResults$counts_query > 0) / 
                                         nrow(motifResults$counts_query), 2),
               'controlFraction' = round(colSums(motifResults$counts_ctrl > 0) / 
                                           nrow(motifResults$counts_ctrl), 2))
    # for each motif pattern, calculate p-value and odds ratio of the motif
    # occurrence in query sequences compared to the control sequences. 
    df <- cbind(df, do.call(rbind, apply(df, 1, function(r) {
      querySeqCount <- as.numeric(r[4])
      ctrlSeqCount <- as.numeric(r[5])
      t <- fisher.test(matrix(data = c(querySeqCount, ctrlSeqCount, 
                                  nrow(motifResults$counts_query) - querySeqCount, 
                                  nrow(motifResults$counts_ctrl) - ctrlSeqCount), 
                         nrow = 2), alternative = 'greater')
      return(data.frame('oddsRatio' = round(t$estimate[[1]], 2), 
                        'pvalue' = t$p.value))
    })))
    return(df)
  } else {
    return(data.frame('patterns' = 'NONE',
                      'queryHits' = 0,
                      'controlHits' = 0,
                      'querySeqs' = 0,
                      'controlSeqs' = 0,
                      'queryFraction' = 0,
                      'controlFraction' = 0, 
                      'oddsRatio' = 1,
                      'pvalue' = 1))
  }
}


#' discoverFeatureSpecificMotifs
#' 
#' This function groups query regions based on their overlap with different
#' transcript features and generates a table of top enriched motif and matching
#' patterns for each given transcript feature type along with some other motif 
#' discovery related statistics.
#' 
#' @param queryRegions GRanges object containing coordinates of input query
#'   regions imported by the \code{\link{importBed}} function
#' @param txdbFeatures A list of GRanges objects where each GRanges object
#'   corresponds to the genomic coordinates of gene features such as promoters,
#'   introns, exons, 5'/3' UTRs and whole transcripts.
#'   This list of GRanges objects are obtained by the function
#'   \code{\link{getTxdbFeaturesFromGRanges}} or \code{\link{getTxdbFeatures}}.
#' @param ... Other arguments passed to \code{\link{runMotifRG}} function.
#'   Important arguments are 'genomeVersion' and motifN. If motifN is bigger
#'   than 1, then multiple motifs will be found but only the top motif will be
#'   plotted.
#' @examples
#' \dontrun{
#' data(gff)
#' data(queryRegions)
#' txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)
#' discoverFeatureSpecificMotifs(queryRegions = queryRegions, 
#' genomeVersion = 'hg19', txdbFeatures = txdbFeatures, 
#' motifN = 1, nCores = 1)}
#' 
#' @return A data.frame object  
#' @export 
discoverFeatureSpecificMotifs <- function(queryRegions, txdbFeatures, ...) {
  results <- lapply(names(txdbFeatures), function(f) {
    message("Looking for motifs in feature:",f)
    featureCoords <- txdbFeatures[[f]]
    #find query regions that overlap the target features
    q <- queryRegions[unique(queryHits(findOverlaps(queryRegions, featureCoords)))]
    
    if(length(q) > 0) {
      motifResults <- runMotifDiscovery(queryRegions = q, ...)
      return(motifResults)
    }
  })
  names(results) <- names(txdbFeatures)
  return(results)
}



#' getPWM
#' 
#' Given a vector of strings of equal width, generate a position-specific 
#' weight matrix based on the frequency of occurrence of the unique letters
#' found in the sequences. 
#' 
#' @param sequences vector of strings of equal widths. 
#' @param letters vector of characters to consider as the an alphabet
#' @examples
#' 
#' sequences = c("GGAGAG", "GAAGAA", "TGAGAA", "GGAGAA", "GAAGAA")
#' getPWM(sequences)
#' 
#' @return A matrix of position-specific-weights.   
#' @export 
getPWM <- function(sequences, letters = c('A', 'C', 'G', 'T')) {
  chars <- strsplit(sequences, '')
  if(length(unique(lengths(chars))) > 1) {
    stop("getPWM: All sequences must be of the same width.")
  }
  m <- do.call(rbind, chars) # matrix of chars 
  # for each position, count occurrences 
  counts <- apply(m, 2, function(y) {
    sapply(letters, function(l) {
      sum(y == l)
    })
  })
  # convert counts to fractions
  pwm <- apply(counts, 2, function(x) x / sum(x))
  return(pwm)
}

library(RCAS)
context("Functions to do motif enrichment analysis")

data(queryRegions)
peaks <- queryRegions[1:1000]

controls <- createControlRegions(queryRegions = peaks)
test_that("Generating control regions to compare with query regions", {
  expect_is(controls, 'GRanges')
  expect_equal(length(controls), length(peaks))
  expect_equal(sum(width(controls) == width(peaks)), length(peaks))
})

test_that("Extracting sequences of a GRanges object from a BSGenome object", {
  expect_is(extractSequences(peaks[1:5], 'hg19'), 'DNAStringSet')
  expect_equal(width(extractSequences(peaks[1:5], 'hg19')), width(peaks[1:5]))
  expect_error(extractSequences(peaks[1:5], 'foo'))
})

motifs <- runMotifDiscovery(queryRegions = queryRegions, genomeVersion = 'hg19', 
                            motifWidth = 4, maxMismatch = 0, 
                            motifN = 1, nCores = 1)
test_that("Motif analysis ... ",{
  expect_is(motifs, 'list')
})

test_that("Motif summary table", {
  expect_equal(length(colnames(getMotifSummaryTable(motifs))), 9)
  expect_equal(colnames(getMotifSummaryTable(motifs))[1], 'patterns')
})

#' plotFeatureBoundaryCoverage
#' 
#' This function is used to create interactive plots displaying 5' and 3'
#' end coverage profiles of given transcript features. 
#' 
#' @param cvgF data.frame object containing 'fiveprime' coverage data returned 
#'   by getFeatureBoundaryCoverage function
#' @param cvgT data.fram object containing 'threeprime' coverage data returned 
#'   by getFeatureBoundaryCoverage function
#' @param featureName character object. This is used to label the axes (e.g.
#'   transcripts, exons)
#' @examples 
#' data(queryRegions)
#' data(gff)
#' txdb <- txdbmaker::makeTxDbFromGRanges(gff)
#' transcriptCoords <- GenomicFeatures::transcripts(txdb)
#' cvgF <- getFeatureBoundaryCoverage (queryRegions = queryRegions,
#'                                     featureCoords = transcriptCoords,
#'                                     flankSize = 100,
#'                                     boundaryType = 'fiveprime',
#'                                     sampleN = 1000)
#'                                     
#' cvgT <- getFeatureBoundaryCoverage (queryRegions = queryRegions,
#'                                     featureCoords = transcriptCoords,
#'                                     flankSize = 100,
#'                                     boundaryType = 'threeprime',
#'                                     sampleN = 1000)
#' p <- plotFeatureBoundaryCoverage(cvgF = cvgF, 
#'                                  cvgT = cvgT, 
#'                           featureName = 'transcript')   
#'                                                           
#' @return a plotly htmlwidget is returned
#' @export 
plotFeatureBoundaryCoverage <- function (cvgF, cvgT, featureName) {

  p1 <- plotly::plot_ly(data = cvgF, x = ~bases, y = ~meanCoverage, 
                        type = 'scatter', mode = 'lines', 
                        name = paste(featureName, "5' end coverage")) %>% 
    add_ribbons(x = ~bases, 
                ymin = cvgF$meanCoverage - cvgF$standardError*1.96, 
                ymax = cvgF$meanCoverage + cvgF$standardError*1.96,
                line = list(color = 'rgba(182, 7, 127, 0.05)'),
                fillcolor = 'rgba(182, 7, 127, 0.22)',
                name = paste(featureName, "5' standard error (95% conf. int.)")) 
  
  p2 <- plotly::plot_ly(data = cvgT, x = ~bases, y = ~meanCoverage, 
                        type = 'scatter', mode = 'lines', 
                        name = paste(featureName, "3' end coverage")) %>% 
    add_ribbons(x = ~bases, 
                ymin = cvgT$meanCoverage - cvgT$standardError*1.96, 
                ymax = cvgT$meanCoverage + cvgT$standardError*1.96, 
                line = list(color = 'rgba(7, 164, 181, 0.05)'),
                fillcolor = 'rgba(7, 164, 181, 0.2)',
                name = paste(featureName, "3' standard error (95% conf. int.)")
    )
  
  p <- plotly::subplot(p1, p2, shareY = TRUE) %>% 
    layout (xaxis = list(title = "Distance (bp) to 5' boundary"), 
            xaxis2 = list(title = "Distance (bp) to 3' boundary"),
            yaxis = list(title = "Mean Coverage Score"),
            legend = list(x = 0, y = 100, orientation = 'h'),
            font = list(size = 14), 
            margin = list(l = 100, b = 100))
  return(p)
}

## ----global_options, include=FALSE--------------------------------------------
knitr::opts_chunk$set(warning = FALSE, message = FALSE, eval = TRUE, fig.width = 7, fig.height = 4.2)

## ----load_libraries, results='hide'-------------------------------------------
library(RCAS)

## ----sample_data--------------------------------------------------------------
library(RCAS)
data(queryRegions) #sample queryRegions in BED format()
data(gff)          #sample GFF file

## ----RCAS_import_data, eval = FALSE-------------------------------------------
# queryRegions <- importBed(filePath = <path to BED file>, sampleN = 10000)
# gff <- importGtf(filePath = <path to GTF file>)

## ----queryGFF-----------------------------------------------------------------
overlaps <- as.data.table(queryGff(queryRegions = queryRegions, gffData = gff))

## ----query_gene_types---------------------------------------------------------
biotype_col <- grep('gene_biotype', colnames(overlaps), value = T)
df <- overlaps[,length(unique(queryIndex)), by = biotype_col]
colnames(df) <- c("feature", "count")
df$percent <- round(df$count / length(queryRegions) * 100, 1)
df <- df[order(count, decreasing = TRUE)]
ggplot2::ggplot(df, aes(x = reorder(feature, -percent), y = percent)) + 
  geom_bar(stat = 'identity', aes(fill = feature)) + 
  geom_label(aes(y = percent + 0.5), label = df$count) + 
  labs(x = 'transcript feature', y = paste0('percent overlap (n = ', length(queryRegions), ')')) + 
  theme_bw(base_size = 14) + 
  theme(axis.text.x = element_text(angle = 90))

## ----getTxdbFeatures----------------------------------------------------------
txdbFeatures <- getTxdbFeaturesFromGRanges(gff)

## ----summarizeQueryRegions----------------------------------------------------
summary <- summarizeQueryRegions(queryRegions = queryRegions, 
                                 txdbFeatures = txdbFeatures)

df <- data.frame(summary)
df$percent <- round((df$count / length(queryRegions)), 3) * 100
df$feature <- rownames(df)
ggplot2::ggplot(df, aes(x = reorder(feature, -percent), y = percent)) + 
  geom_bar(stat = 'identity', aes(fill = feature)) + 
  geom_label(aes(y = percent + 3), label = df$count) + 
  labs(x = 'transcript feature', y = paste0('percent overlap (n = ', length(queryRegions), ')')) + 
  theme_bw(base_size = 14) + 
  theme(axis.text.x = element_text(angle = 90))

## ----getTargetedGenesTable----------------------------------------------------
dt <- getTargetedGenesTable(queryRegions = queryRegions, 
                           txdbFeatures = txdbFeatures)
dt <- dt[order(transcripts, decreasing = TRUE)]

knitr::kable(dt[1:10,])

## ----transcriptBoundaryCoverage-----------------------------------------------
cvgF <- getFeatureBoundaryCoverage(queryRegions = queryRegions, 
                                   featureCoords = txdbFeatures$transcripts, 
                                   flankSize = 1000, 
                                   boundaryType = 'fiveprime', 
                                   sampleN = 10000)
cvgT <- getFeatureBoundaryCoverage(queryRegions = queryRegions, 
                                   featureCoords = txdbFeatures$transcripts, 
                                   flankSize = 1000, 
                                   boundaryType = 'threeprime', 
                                   sampleN = 10000)

cvgF$boundary <- 'fiveprime'
cvgT$boundary <- 'threeprime'

df <- rbind(cvgF, cvgT)

ggplot2::ggplot(df, aes(x = bases, y = meanCoverage)) + 
  geom_ribbon(fill = 'lightgreen', 
              aes(ymin = meanCoverage - standardError * 1.96, 
                  ymax = meanCoverage + standardError * 1.96)) + 
 geom_line(color = 'black') + 
 facet_grid( ~ boundary) + theme_bw(base_size = 14) 


## ----coverageprofilelist, fig.height=6----------------------------------------
cvgList <- calculateCoverageProfileList(queryRegions = queryRegions, 
                                       targetRegionsList = txdbFeatures, 
                                       sampleN = 10000)

ggplot2::ggplot(cvgList, aes(x = bins, y = meanCoverage)) + 
  geom_ribbon(fill = 'lightgreen', 
              aes(ymin = meanCoverage - standardError * 1.96, 
                  ymax = meanCoverage + standardError * 1.96)) + 
 geom_line(color = 'black') + theme_bw(base_size = 14) +
 facet_wrap( ~ feature, ncol = 3) 

## ----motif_analysis-----------------------------------------------------------
motifResults <- runMotifDiscovery(queryRegions = queryRegions, 
                           resizeN = 15, sampleN = 10000,
                           genomeVersion = 'hg19', motifWidth = 6,
                           motifN = 1, nCores = 1)

seqLogo::seqLogo(getPWM(motifResults$matches_query[[1]]))

## ----motif_analysis_table-----------------------------------------------------
summary <- getMotifSummaryTable(motifResults)
knitr::kable(summary)

## ----GO analysis--------------------------------------------------------------
targetedGenes <- unique(overlaps$gene_id)

res <- RCAS::findEnrichedFunctions(targetGenes = targetedGenes, species = 'hsapiens')
res <- res[order(res$p_value),]
resGO <- res[grep('GO:BP', res$source),]
knitr::kable(subset(resGO[1:10,], select = c('p_value', 'term_name', 'source')))

#' importBedFiles
#'
#' This function is a wrapper that uses \code{RCAS::importBed()} function to 
#' import BED files as a GRangesList object
#' 
#' @param filePaths A vector of paths to one or more BED files
#' @param ... Other parameters passed to RCAS::importBed and
#'   rtracklayer::import.bed function
#' @return A \code{GRangesList} object containing the coordinates of the 
#'   intervals from multiple input BED files
#'   
#' @examples
#' input1 <- system.file("extdata", "testfile.bed", package='RCAS') 
#' input2 <- system.file("extdata", "testfile2.bed", package='RCAS') 
#' bedData <- importBedFiles(filePaths = c(input1, input2), 
#' keepStandardChr = TRUE) 
#' # when importing multiple bed files with different column names, it 
#' # is required to pass the common column names to be parsed from the 
#' # bed files
#' bedData <- importBedFiles(filePaths = c(input1, input2),
#'                  colnames = c('chrom', 'start', 'end', 'strand'))
#' 
#' @importFrom GenomicRanges GRangesList
#' @importFrom pbapply pbsapply
#' @export
importBedFiles <- function(filePaths, ...) {
  bedData <- GenomicRanges::GRangesList(pbapply::pbsapply(filePaths, function(f) {
    RCAS::importBed(filePath = f, debug = FALSE, ...)
  }, USE.NAMES = TRUE))
  names(bedData) <- gsub(pattern = '.bed$', 
                         replacement = '', 
                         x = basename(names(bedData)), 
                         ignore.case = TRUE)
  return(bedData)
}


#' summarizeQueryRegionsMulti
#' 
#' This function is a wrapper function to run RCAS::summarizeQueryRegions 
#' multiple times, which is useful to get a matrix of overlap counts between a 
#' list of BED files with a txdbFeatures extracted from GTF file
#' 
#' @param queryRegionsList GRangesList object imported from multiple BED files
#'   using \code{importBedFiles} function
#' @param txdbFeatures List of GRanges objects - outputs of 
#'   \code{getTxdbFeaturesFromGRanges} and \code{getTxdbFeatures} functions
#' @param nodeN Positive integer value that denotes the number of cpus to use
#'   for parallel processing (default: 1)
#' @return A list consisting of two data.frame objects: one for raw overlap 
#'   counts and one for percentage of overlap counts (raw overlap counts divided
#'   by the number of query regions in the corresponding BED file)
#' @examples
#' data(gff)
#' data(queryRegions)
#' queryRegionsList <- GenomicRanges::GRangesList(queryRegions, queryRegions)
#' names(queryRegionsList) <- c('q1', 'q2')
#' txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)
#' summaryMatrix <- summarizeQueryRegionsMulti(queryRegionsList = queryRegionsList,
#'                                  txdbFeatures = txdbFeatures)
#'
#' @importFrom parallel makeCluster
#' @importFrom parallel clusterExport
#' @importFrom parallel stopCluster
#' @importFrom pbapply pbsapply 
#' @export
summarizeQueryRegionsMulti <- function(queryRegionsList, txdbFeatures, nodeN = 1) {
  cl <- parallel::makeCluster(nodeN)
  parallel::clusterExport(cl = cl, 
                          varlist = c('txdbFeatures', 'summarizeQueryRegions'), 
                          envir = environment())
  summaryRaw <- pbapply::pbsapply(queryRegionsList, 
                                  function(x) {
                                    summarizeQueryRegions(x, txdbFeatures)
                                    }, cl = cl)
  rownames(summaryRaw) <- c(names(txdbFeatures), 'NoFeatures')
  parallel::stopCluster(cl)
  return(t(summaryRaw))
}

#' getFeatureBoundaryCoverageMulti
#' 
#' This function is a wrapper function to run RCAS::getFeatureBoundaryCoverage 
#' multiple times, which is useful to get coverage signals across different 
#' kinds of transcript features for a given list of bed files imported as a 
#' GRangesList object.
#' 
#' @param bedData GRangesList object imported from multiple BED files 
#'   using \code{importBedFiles} function
#' @param txdbFeatures List of GRanges objects - outputs of 
#'   \code{getTxdbFeaturesFromGRanges} and \code{getTxdbFeatures} functions
#' @param sampleN (default=10000) Positive integer value that is used to 
#'   randomly down-sample the target feature coordinates to improve the runtime.
#'   Set to 0 to avoid downsampling.
#'   
#' @return A data.frame object with coverage data at three prime and five prime
#'   boundaries of a list of transcript features
#' @examples
#' data(gff)
#' data(queryRegions)
#' queryRegionsList <- GenomicRanges::GRangesList(queryRegions, queryRegions)
#' names(queryRegionsList) <- c('q1', 'q2')
#' txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)
#' getFeatureBoundaryCoverageMulti(queryRegionsList, txdbFeatures, sampleN = 500)
#' 
#' @importFrom pbapply pblapply
#' @export
getFeatureBoundaryCoverageMulti <- function(bedData, 
                                            txdbFeatures, 
                                            sampleN = 10000) {
  results <- pblapply(X = names(bedData),
    FUN = function(s) {
      queryCoords <- bedData[[s]]
      cvg <- lapply(names(txdbFeatures),
                    function(featureType) {
                      featureCoords <- txdbFeatures[[featureType]]
                      cvgF <- getFeatureBoundaryCoverage(
                        queryRegions = queryCoords,
                        featureCoords = featureCoords,
                        sampleN = sampleN,
                        boundaryType = 'fiveprime')
                      
                      cvgT <- getFeatureBoundaryCoverage(
                        queryRegions = queryCoords,
                        featureCoords = featureCoords,
                        sampleN = sampleN,
                        boundaryType = 'threeprime')
                      
                      cvgF$boundary <- 'fiveprime'
                      cvgT$boundary <- 'threeprime'
                      
                      df <- rbind(cvgF, cvgT)
                      df$feature <- featureType
                      return(df)
                      })
      cvg <- do.call(rbind, cvg)
      cvg$sampleName <- s
      return(cvg)
      })
  return(do.call(rbind, results))
}


#' getIntervalOverlapMatrix
#' 
#' This function is used to obtain a binary matrix of overlaps between a list of
#' GRanges objects (GRangesList object) and a target GRanges object. The
#' resulting matrix has N rows where N is the number of intervals in the target
#' GRanges object and M columns where M is the number GRanges objects in the
#' query GRangesList object.
#' 
#' @param queryRegionsList A GRangesList object
#' @param targetRegions A GRanges object
#' @param targetRegionNames Optional vector of names to be used as rownames in
#'   the resulting matrix. The vector indices must correspond to the intervals
#'   in targetRegions object.
#' @param nodeN Positive integer value to use one or more cpus for parallel
#'   computation (default: 1)
#' @return A binary matrix object consisting of number of rows equal to the
#'   number of intervals in targetRegions object, and number of columns equal to
#'   the number of GRanges objects available in the queryRegionsList object.
#' @examples 
#' data(gff)
#' input1 <- system.file("extdata", "testfile.bed", package='RCAS') 
#' input2 <- system.file("extdata", "testfile2.bed", package='RCAS') 
#' bedData <- RCAS::importBedFiles(filePaths = c(input1, input2)) 
#' M <- RCAS::getIntervalOverlapMatrix(
#' queryRegionsList = bedData, 
#' targetRegions = gff[gff$type == 'gene',][1:100], 
#' targetRegionNames = gff[gff$type == 'gene',][1:100]$gene_name)
#' @export
getIntervalOverlapMatrix <- function(queryRegionsList, targetRegions, targetRegionNames = NULL, nodeN = 1) {
  
  if(!is.null(targetRegionNames) & length(targetRegions) != length(targetRegionNames)) {
    stop("The sizes of targetRegions and targetRegionNames must be equal\n")
  }
  cl <- parallel::makeCluster(nodeN)
  parallel::clusterExport(cl = cl, 
                          varlist = c('targetRegions'), 
                          envir = environment())
  summaryRaw <-  pbapply::pblapply(queryRegionsList[1], 
                                  function(x) {
                                    myOverlaps <- GenomicRanges::findOverlaps(targetRegions, x)
                                    unique(S4Vectors::queryHits(myOverlaps))
                                  }, cl = cl)
  parallel::stopCluster(cl)
  
  rowN <- length(targetRegions)
  colN <- length(queryRegionsList)
  
  M <- do.call(cbind, lapply(X = names(summaryRaw), 
              FUN = function(x) {
                v <- rep(0, rowN) 
                v[summaryRaw[[x]]] <- 1
                v <- data.frame(v)
                colnames(v) <- x  
                return(v)
              }))
  if(!is.null(targetRegionNames)) {
    rownames(M) <- targetRegionNames
  }
  return(M)
}


#' runReportMetaAnalysis
#' 
#' Generate a stand-alone HTML report with interactive figures and tables from a
#' pre-calculated RCAS database (using RCAS::createDB) to compare multiple 
#' samples.
#' 
#' @param dbPath Path to the sqlite database generated by RCAS::createDB
#' @param sampleTablePath A tab-separated file with two columns (no rownames) 
#'   header 1: sampleName, header 2: sampleGroup
#' @param outDir Path to the output directory. (default: current working 
#'   directory)
#' @param outFile Name of the output HTML report (by default, the base name of
#'   sampleTablePath value is used to create a name for the HTML report)
#' @param quiet boolean value (default: FALSE). If set to TRUE, progress bars 
#'   and chunk labels will be suppressed while knitting the Rmd file.
#' @param selfContained boolean value (default: TRUE). By default, the generated
#'   html file will be self-contained, which means that all figures and tables 
#'   will be embedded in a single html file with no external dependencies (See 
#'   rmarkdown::html_document)
#' @return An html generated using rmarkdown/knitr/pandoc that contains 
#'   interactive figures, tables, and text that provide an overview of the 
#'   experiment
#' @examples
#' dbPath <- system.file("extdata", "hg19.RCASDB.sqlite",
#' package='RCAS') 
#' 
#' #Hint: use RCAS::summarizeDatabaseContent to see which samples have processed
#' #data in the database. 
#' summarizeDatabaseContent(dbPath = dbPath)
#' 
#' #Create a data table for samples and their groups sampleGroup field is used
#' #to group replicates of #the same sample into one group in visualizations. 
#' #Any arbitrary name can be used for sampleGroup field. However, entries in
#' #the sampleName field must be available in the queried database
#' sampleData <- data.frame('sampleName' = c('FUS', 'FMR1'), 
#'     'sampleGroup' = c('FUS', 'FMR1'), stringsAsFactors = FALSE) 
#' write.table(sampleData, 'sampleDataTable.tsv', sep = '\t', 
#'     quote =FALSE, row.names = FALSE)
#' #Use the generated database to run a report 
#' runReportMetaAnalysis(dbPath = 'hg19.RCASDB.sqlite', 
#'     sampleTablePath = 'sampleDataTable.tsv')
#' @import rmarkdown
#' @import knitr
#' @importFrom plotly ggplotly
#' @import ggplot2
#' @importFrom pheatmap pheatmap
#' @importFrom cowplot plot_grid
#' @import DT
#' @export
runReportMetaAnalysis <- function(dbPath = 'RCAS.sqlite',
                                  sampleTablePath,
                                  outDir = getwd(),
                                  outFile = NULL,
                                  quiet = FALSE,
                                  selfContained = TRUE) {
  
  reportFile <- system.file("reporting_scripts", 
                            "reportMetaAnalysis.Rmd", package='RCAS')
  headerFile <- system.file("reporting_scripts", 
                            "header.html", package='RCAS')
  footerFile <- system.file("reporting_scripts", 
                            "footer.html", package='RCAS')
  if(is.null(outFile)) {
    outFile <- paste0(basename(sampleTablePath), '.RCAS.metaAnalysisReport.html')
  } 
  rmarkdown::render(
    input = reportFile, 
    output_dir = outDir,
    intermediates_dir = outDir,
    output_file = outFile,
    output_format = rmarkdown::html_document(
      toc = TRUE,
      toc_float = TRUE,
      theme = 'simplex',
      number_sections = TRUE,
      includes = rmarkdown::includes(in_header = headerFile, 
                                     after_body = footerFile), 
      self_contained = selfContained
    ),
    params = list(dbPath = dbPath,
                  sampleTablePath = sampleTablePath,
                  workdir = outDir),
    quiet = quiet
  )
}

#' Sample GFF file imported as a GRanges object
#'
#' This dataset contains genomic annotation data from Ensembl version 75 for
#' Homo sapiens downloaded from Ensembl. The GFF file is imported via the
#' \code{importGtf} function and a subset of the data is selected by choosing
#' features found on 'chr1'.
#'
#' @return A GRanges object
#' @format GRanges object with 238010 ranges and 16 metadata columns
#' @source \url{ftp://ftp.ensembl.org/pub/release-75/gtf/homo_sapiens/Homo_sapiens.GRCh37.75.gtf.gz}
"gff"

#' Sample BED file imported as a GRanges object
#'
#' This dataset contains a randomly selected sample of human LIN28A protein
#' binding sites detected by HITS-CLIP analysis downloaded from DoRina database
#' (LIN28A HITS-CLIP hESCs (Wilbert 2012)). The BED file is imported via the
#' \code{importBed} function and a subset of the data is selected by randomly
#' choosing 10000 regions.
#'
#' @return A GRanges object
#' @format GRanges object with 10000 ranges and 2 metadata columns
#' @source \url{http://dorina.mdc-berlin.de/regulators}
"queryRegions"

#' findEnrichedFunctions
#' 
#' Find enriched functional terms among the genes that overlap
#' the regions of interest. 
#' 
#' This function is basically a call to gprofiler2::gost function. 
#' It is here to serve as a replacement for other deprecated functional
#' enrichment functions. 
#' 
#' @examples 
#' data(gff)
#' data(queryRegions)
#' 
#' overlaps <- queryGff(queryRegions, gff)
#' res <- findEnrichedFunctions(unique(overlaps$gene_id), 'hsapiens')
#' 
#' @param targetGenes Vector of Ensembl gene ids or gene names 
#' @param species First letter of genus + species name: e.g. hsapiens
#' @param ... Other arguments to be passed to gprofiler2::gost 
#' @importFrom gprofiler2 gost
#' @export
findEnrichedFunctions <- function(targetGenes, species, ...) {
 res <- gprofiler2::gost(query = targetGenes, organism = species)
 return(res$result)
}

#' runTopGO
#' 
#' This function is deprecated. Use findEnrichedFunctions instead. 
#' @export
runTopGO <- function () {
  .Deprecated('findEnrichedFunctions')
}

#' parseMsigdb
#'
#' This function is deprecated. For functional enrichment 
#' analysis, use findEnrichedFunctions. 
#' 
#' @export
parseMsigdb <- function(){
  .Deprecated('findEnrichedFunctions.')
}

#' Print MSIGDB Dataset to a file
#' 
#' This function is deprecated. For functional enrichment 
#' analysis, use findEnrichedFunctions. 
#' @export
printMsigdbDataset = function(){
  .Deprecated('findEnrichedFunctions.')
}

#' retrieveOrthologs
#' 
#' This function is deprecated. For functional enrichment 
#' analysis, use findEnrichedFunctions. 
#'
#' @export
retrieveOrthologs <- function(){
  .Deprecated('findEnrichedFunctions')
}

#' createOrthologousMsigdbDataset
#' This function is deprecated. For functional enrichment 
#' analysis, use findEnrichedFunctions. 
#' @export
createOrthologousGeneSetList <- function() {
  .Deprecated('findEnrichedFunctions')
}

#' runGSEA
#' 
#' This function is deprecated. For functional enrichment 
#' analysis, use findEnrichedFunctions. 
#' 
#' @export
runGSEA <- function () {
  .Deprecated('findEnrichedFunctions')
}

.datatable.aware=TRUE

#' importGtf
#'
#' This function uses \code{rtracklayer::import.gff()} function to import genome
#' annoatation data from an Ensembl gtf file
#'
#' @param filePath Path to a GTF file
#' @param saveObjectAsRds TRUE/FALSE (default:TRUE). If it is set to TRUE, a
#'   GRanges object will be created and saved in RDS format
#'   (<filePath>.granges.rds) so that importing can re-use this .rds file in
#'   next run.
#' @param readFromRds TRUE/FALSE (default:TRUE). If it is set to TRUE,
#'   annotation data will be imported from previously generated .rds file
#'   (<filePath>.granges.rds).
#' @param overwriteObjectAsRds TRUE/FALSE (default:FALSE). If it is set to TRUE,
#'   existing .rds file (<filePath>.granges.rds) will overwritten.
#' @param keepStandardChr TRUE/FALSE (default:TRUE). If it is set to TRUE,
#'   \code{seqlevelsStyle} will be converted to 'UCSC' and
#'   \code{keepStandardChromosomes} function  will be applied to only keep data
#'   from the standard chromosomes.
#' @param ... Other arguments passed to rtracklayer::import.gff function
#' @return A \code{GRanges} object containing the coordinates of the annotated
#'   genomic features in an input GTF file
#'
#' @examples
#' #import the data and write it into a .rds file
#' \dontrun{
#' importGtf(filePath='./Ensembl75.hg19.gtf')
#' }
#' #import the data but don't save it as RDS
#' \dontrun{
#' importGtf(filePath='./Ensembl75.hg19.gtf', saveObjectAsRds = FALSE)
#' }
#' #import the data and overwrite the previously generated
#' \dontrun{
#' importGtf(filePath='./Ensembl75.hg19.gtf', overwriteObjectAsRds = TRUE)
#' }
#'
#' @importFrom rtracklayer import.gff
#' @importFrom GenomeInfoDb seqlevelsStyle
#' @importFrom GenomeInfoDb keepStandardChromosomes
#' @export
importGtf <- function (filePath,  saveObjectAsRds = TRUE, readFromRds = TRUE,
                       overwriteObjectAsRds = FALSE, keepStandardChr = TRUE, ...) {

  rdsFilePath <- paste0(filePath, ".granges.rds")

  if (readFromRds == TRUE && file.exists(rdsFilePath)){
    message('Reading existing granges.rds object from ',rdsFilePath)
    gff <- readRDS(rdsFilePath)
  } else {
    message('importing gtf file from ', filePath)
    gff <- rtracklayer::import.gff(filePath, ...)
  }

  if (!is.null(gff)) {
    if (keepStandardChr == TRUE) {
      message('Keeping standard chromosomes only')
      GenomeInfoDb::seqlevelsStyle(gff) = 'UCSC'
      gff = GenomeInfoDb::keepStandardChromosomes(gff, pruning.mode = 'coarse')
    }

    if (saveObjectAsRds == TRUE) {
      if (!file.exists(rdsFilePath)){
        message('Saving gff object in RDS file at ',rdsFilePath)
        saveRDS(gff, file=rdsFilePath)
      } else {
        if (overwriteObjectAsRds == TRUE) {
          message('Overwriting gff object in RDS file at ',rdsFilePath)
          saveRDS(gff, file=rdsFilePath)
        } else {
          message('File ',rdsFilePath,' already exists.
                  Use overwriteObjectAsRds = TRUE to overwrite the file')
        }
      }
    return (gff)
    }
  } else{
    stop("Couldn't import the gtf from",filePath,"\n")
  }
}


#' importBed
#'
#' This function uses \code{rtracklayer::import.bed()} function to import BED
#' files
#'
#' @param filePath Path to a BED file
#' @param sampleN A positive integer value. The number of intervals in the input
#'   BED file are randomly downsampled to include intervals as many as
#'   \code{sampleN}. The input will be downsampled only if this value is larger
#'   than zero and less than the total number of input intervals.
#' @param keepStandardChr TRUE/FALSE (default:TRUE). If set to TRUE, will
#'   convert the \code{seqlevelsStyle} to 'UCSC' and apply
#'   \code{keepStandardChromosomes} function to only keep data from the standard
#'   chromosomes
#' @param debug TRUE/FALSE (default:TRUE). Set to FALSE to turn off messages
#' @param ... Other arguments passed to rtracklayer::import.bed function
#' @return A \code{GRanges} object containing the coordinates of the intervals
#'   from an input BED file
#'
#' @examples
#' input <- system.file("extdata", "testfile.bed", package='RCAS')
#' importBed(filePath = input, keepStandardChr = TRUE)
#'
#' @importFrom rtracklayer import.bed
#' @importFrom GenomeInfoDb seqlevelsStyle
#' @importFrom GenomeInfoDb keepStandardChromosomes
#' @export
importBed <- function (filePath, sampleN = 0, keepStandardChr = TRUE, debug = TRUE, ...) {

  if (file.exists(filePath)) {
    data = rtracklayer::import.bed(filePath, ...)
    if(debug == TRUE) {
      message('Processing ',filePath)
    }
    if (keepStandardChr == TRUE) {
      GenomeInfoDb::seqlevelsStyle(data) <- 'UCSC'
      if(debug == TRUE) {
        message('Keeping standard chromosomes only')
      }
      data <- GenomeInfoDb::keepStandardChromosomes(data, pruning.mode = 'coarse')
    }

    if (sampleN > 0 && sampleN < length(data)) {
      if(debug == TRUE) {
        message ('Downsampling intervals to size:',sampleN)
      }
      data <- data[sample(x = 1:length(data), size = sampleN)]
    }
    return(data)
  } else {
    stop("Cannot import BED file.
         It does not exist at the given location:",filePath,"\n")
  }
}


#' getTxdbFeatures
#' 
#' This function is deprecated. Use getTxdbFeaturesFromGRanges instead. 
#' 
#' @export
getTxdbFeatures <- function () {
  .Deprecated("getTxdbFeaturesFromGRanges")
}

#' getTxdbFeaturesFromGRanges
#'
#' This function takes as input a GRanges object that contains GTF file contents
#' (e.g from the output of \code{importGtf} function). 
#' Then extracts the coordinates of gene features
#' such as promoters, introns, exons, 5'/3' UTRs and
#' whole transcripts.
#'
#' @param gffData A GRanges object imported by \code{importGtf} function
#'
#' @examples
#' data(gff)
#' txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)
#'
#' @return A list of GRanges objects
#'
#' @importFrom GenomicFeatures transcripts exonsBy cdsBy intronsByTranscript
#' fiveUTRsByTranscript threeUTRsByTranscript promoters
#' @importFrom txdbmaker makeTxDbFromGRanges
#' @import GenomicRanges
#' @importFrom BiocGenerics unlist
#' @export
getTxdbFeaturesFromGRanges <- function (gffData) {

  txdb <- txdbmaker::makeTxDbFromGRanges(gffData)

  transcripts <- GenomicFeatures::transcripts(txdb)
  m <- match(transcripts$tx_name, gffData$transcript_id)
  transcripts$gene_name <- gffData[m]$gene_name

  tmp <- GenomicFeatures::exonsBy(x = txdb, by = "tx", use.names = TRUE)
  exons <- BiocGenerics::unlist(tmp)
  exons$tx_name <- names(exons)
  exons$gene_name <- gffData[match(names(exons), gffData$transcript_id)]$gene_name

  tmp <- GenomicFeatures::intronsByTranscript(x = txdb, use.names = TRUE)
  introns <- BiocGenerics::unlist(tmp)
  introns$tx_name <- names(introns)
  m <- match(names(introns), gffData$transcript_id)
  introns$gene_name <- gffData[m]$gene_name

  promoters <- GenomicFeatures::promoters(txdb)
  m <- match(promoters$tx_name, gffData$transcript_id)
  promoters$gene_name <- gffData[m]$gene_name

  #fiveUTRsByTranscript will return exonic regions of UTRs. These exonic regions
  #must be turned into a single region containing both exons and introns using
  #the range function we can get min-start and max-end of all UTR regions per
  #transcript
  tmp <- range(GenomicFeatures::fiveUTRsByTranscript(x = txdb, use.names = TRUE))
  fiveUTRs <- BiocGenerics::unlist(tmp)
  fiveUTRs$tx_name <- names(fiveUTRs)
  m <- match(names(fiveUTRs), gffData$transcript_id)
  fiveUTRs$gene_name <- gffData[m]$gene_name

  #threeUTRsByTranscript will return exonic regions of UTRs. These exonic regions
  #must be turned into a single region containing both exons and introns using
  #the range function we can get min-start and max-end of all UTR regions per
  #transcript
  tmp <- range(GenomicFeatures::threeUTRsByTranscript(x = txdb, use.names = TRUE))
  threeUTRs <- BiocGenerics::unlist(tmp)
  threeUTRs$tx_name <- names(threeUTRs)
  m <- match(names(threeUTRs), gffData$transcript_id)
  threeUTRs$gene_name <- gffData[m]$gene_name

  tmp <- GenomicFeatures::cdsBy(x = txdb, by = "tx", use.names = TRUE)
  cds <- BiocGenerics::unlist(tmp)
  cds$tx_name <- names(cds)
  cds$gene_name <- gffData[match(names(cds), gffData$transcript_id)]$gene_name

  txdbFeatures = list(
    'transcripts' = transcripts,
    'exons'       = exons,
    'promoters'   = promoters,
    'fiveUTRs'    = fiveUTRs,
    'introns'     = introns,
    'cds'         = cds,
    'threeUTRs'   = threeUTRs
  )
  return(txdbFeatures)
}

#' @importFrom data.table data.table
#' @importFrom S4Vectors queryHits
#' @importFrom S4Vectors subjectHits
#' @import GenomicRanges
processHits <- function(queryRegions, tx, type) {
  overlaps <- GenomicRanges::findOverlaps(queryRegions, tx)
  overlapsQuery <- queryRegions[S4Vectors::queryHits(overlaps)]
  overlapsTX <- tx[S4Vectors::subjectHits(overlaps)]
  overlapsTX$overlappingQuery <- paste(GenomicRanges::seqnames(overlapsQuery),
                                       GenomicRanges::start(overlapsQuery),
                                       GenomicRanges::end(overlapsQuery),
                                       GenomicRanges::strand(overlapsQuery),
                                       sep=':')
  dt <- data.table::data.table('tx_name' = overlapsTX$tx_name,
                               'query' = overlapsTX$overlappingQuery)
  query <- 'query'
  summary <- dt[,length(unique(query)), by='tx_name']
  colnames(summary) <- c('tx_name', type)
  return(summary)
}


#' getTargetedGenesTable
#'
#' This function provides a list of genes which are targeted by query regions
#' and their corresponding numbers from an input BED file. Then, the hits are
#' categorized by the gene features such as promoters, introns, exons,
#' 5'/3' UTRs and whole transcripts.
#'
#' @param queryRegions GRanges object containing coordinates of input query
#'   regions imported by the \code{\link{importBed}} function
#' @param txdbFeatures A list of GRanges objects where each GRanges object
#'   corresponds to the genomic coordinates of gene features such as promoters,
#'   introns, exons, 5'/3' UTRs and whole transcripts.
#'   This list of GRanges objects are obtained by the function
#'   \code{\link{getTxdbFeaturesFromGRanges}} or \code{\link{getTxdbFeatures}}.
#'
#' @examples
#' data(gff)
#' data(queryRegions)
#' txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)
#' featuresTable <- getTargetedGenesTable(queryRegions = queryRegions,
#'                                        txdbFeatures = txdbFeatures)
#'
#' #or
#' \dontrun{
#' txdb <- txdbmaker::makeTxDbFromGRanges(gff)
#' txdbFeatures <- getTxdbFeatures(txdb)
#' featuresTable <- getTargetedGenesTable(queryRegions = queryRegions,
#'                                        txdbFeatures = txdbFeatures)
#'                                        }
#' @return A data.frame object where rows correspond to genes and columns
#'   correspond to gene features
#'
#' @importFrom data.table setkey
#' @importFrom S4Vectors Reduce
#' @export
getTargetedGenesTable <- function (queryRegions, txdbFeatures) {

  tbls <- lapply(X = seq_along(txdbFeatures),
                 FUN = function(i) {
                   processHits(queryRegions = queryRegions,
                               tx = txdbFeatures[[names(txdbFeatures)[i]]],
                               type = names(txdbFeatures)[i])})

  tbls <- lapply(tbls, function(i) data.table::setkey(i, 'tx_name'))
  merged <- S4Vectors::Reduce(function(...) merge(..., all = TRUE), tbls)
  merged[is.na(merged)] <- 0
  return(merged)
}

#' summarizeQueryRegions
#'
#' This function counts number of query regions that overlap with different
#' types of gene features.
#'
#' @param queryRegions GRanges object imported from a BED file using
#'   \code{importBed} function
#' @param txdbFeatures List of GRanges objects - outputs of
#'   \code{getTxdbFeaturesFromGRanges} and \code{getTxdbFeatures} functions
#' @return A data frame with two columns where first column holds features and
#'   second column holds corresponding counts
#' @examples
#' data(gff)
#' data(queryRegions)
#' txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)
#' summary <- summarizeQueryRegions(queryRegions = queryRegions,
#'                                  txdbFeatures = txdbFeatures)
#'
#' @import GenomicRanges
#' @importFrom S4Vectors queryHits
#' @export
summarizeQueryRegions <- function(queryRegions, txdbFeatures) {
  summarize <- function (x) {
    myOverlaps <- GenomicRanges::findOverlaps(queryRegions, x)
    unique(S4Vectors::queryHits(myOverlaps))
  }
  results <- lapply(X = txdbFeatures, FUN = summarize)
  results$NoFeatures <- setdiff(c(1:length(queryRegions)), 
                                as.vector(do.call(c, results)))
  
  df <- t(data.frame(lapply(results, length)))
  colnames(df) <- c('count')
  return(df)
}

#' queryGff
#'
#' This function checks overlaps between the regions in input query and in
#' reference. Input query should be in BED format and reference should be in GFF
#' format. Both data are imported as GRanges object.
#'
#' @param queryRegions GRanges object imported from a BED file using
#'   \code{importBed} function
#' @param gffData GRanges object imported from a GTF file using \code{importGtf}
#'   function
#' @return a GRanges object (a subset of input gff) with an additional column
#'   'overlappingQuery' that contains the coordinates of query regions that
#'   overlap the target annotation features
#'
#' @examples
#' data(queryRegions)
#' data(gff)
#' overlaps <- queryGff(queryRegions = queryRegions, gffData = gff)
#'
#' @import GenomicRanges
#' @importFrom S4Vectors queryHits
#' @importFrom S4Vectors subjectHits
#' @export
queryGff <- function(queryRegions, gffData) {
  #find all overlapping pairs of intervals between gff features and BED file
  overlaps = GenomicRanges::findOverlaps(queryRegions, gffData)
  overlapsQuery = queryRegions[S4Vectors::queryHits(overlaps)]
  overlapsGff = gffData[S4Vectors::subjectHits(overlaps)]
  
  # prepare some data for the overlapping query region 
  # to keep along with overlapping gff features 
  query_mcols <- mcols(overlapsQuery)
  colnames(query_mcols) <- paste0('query_', colnames(query_mcols))
  
  queryData <- cbind(
    data.frame('queryIndex' = S4Vectors::queryHits(overlaps), 
               'queryRange' = paste0(GenomicRanges::seqnames(overlapsQuery), ':', 
                                  GenomicRanges::start(overlapsQuery), '-', 
                                  GenomicRanges::end(overlapsQuery), ':',
                                  GenomicRanges::strand(overlapsQuery))),
    query_mcols)
  
  # update mcols portion of overlapping gff feature data           
  mcols(overlapsGff) <- cbind(mcols(overlapsGff), queryData)
  return (overlapsGff)
}


#' getFeatureBoundaryCoverage
#' 
#' This function extracts the flanking regions of 5' and 3' boundaries of a 
#' given set of genomic features and computes the per-base coverage of query 
#' regions across these boundaries.
#' 
#' @param queryRegions GRanges object imported from a BED file using 
#'   \code{importBed} function
#' @param featureCoords GRanges object containing the target feature coordinates
#' @param flankSize Positive integer that determines the number of base pairs to
#'   extract around a given genomic feature boundary
#' @param boundaryType (Options: fiveprime or threeprime). Denotes which side of
#'   the feature's boundary is to be profiled.
#' @param sampleN A positive integer value less than the total number of featuer
#'   coordinates that determines whether the target feature coordinates should 
#'   be randomly downsampled. If set to 0, no downsampling will happen. If
#' @return a data frame containin three columns. 1. fivePrime: Coverage at 5' 
#'   end of features 2. threePrime: Coverage at 3' end of features; 3. bases: 
#'   distance (in bp) to the boundary
#'
#' @examples
#' data(queryRegions)
#' data(gff)
#' txdb <- txdbmaker::makeTxDbFromGRanges(gff)
#' transcriptCoords <- GenomicFeatures::transcripts(txdb)
#' transcriptEndCoverage <- getFeatureBoundaryCoverage (
#'                                      queryRegions = queryRegions,
#'                                     featureCoords = transcriptCoords,
#'                                     flankSize = 100,
#'                                     boundaryType = 'threeprime',
#'                                     sampleN = 1000)
#' @import GenomicRanges
#' @importFrom genomation ScoreMatrix
#' @export
getFeatureBoundaryCoverage <- function (queryRegions,
                                         featureCoords,
                                         flankSize = 500,
                                         boundaryType, 
                                         sampleN = 0) {
  
  if (sampleN > 0 && sampleN < length(featureCoords)) {
    featureCoords <- sort(featureCoords[sample(length(featureCoords), sampleN)])
  }
  
  if (boundaryType == 'fiveprime') {
    flanks <- GenomicRanges::flank(x = featureCoords,
                                   width = flankSize,
                                   start = TRUE,
                                   both = TRUE)
  } else if (boundaryType == 'threeprime') { 
    flanks <- GenomicRanges::flank(x = featureCoords,
                                   width = flankSize,
                                   start = FALSE,
                                   both = TRUE)
  } else {
    stop ("please indicate either threeprime or fiveprime for boundary type\n")
  }
  
  sm <- genomation::ScoreMatrix(target = queryRegions,
                                windows = flanks,
                                strand.aware = TRUE)
  
  return (data.frame('bases' = c(-flankSize:(flankSize-1)), 
                     'meanCoverage' = colMeans(sm), 
                     'standardError' = apply(sm, 2, plotrix::std.error))
  )
}
  

#' getFeatureBoundaryCoverageBin
#'
#' This function extracts the flanking regions of 5' and 3' boundaries of a
#' given set of genomic features, splits them into 100 equally sized bins and
#' computes the per-bin coverage of query regions across these boundaries.
#'
#' @param queryRegions GRanges object imported from a BED file using
#'   \code{importBed} function
#' @param featureCoords GRanges object containing the target feature coordinates
#' @param flankSize Positive integer that determines the number of base pairs to
#'   extract around a given genomic feature boundary
#' @param sampleN A positive integer value less than the total number of featuer
#'   coordinates that determines whether the target feature coordinates should
#'   be randomly downsampled. If set to 0, no downsampling will happen. If
#' @return a data frame containin three columns. 1. fivePrime: Coverage at 5'
#'   end of features 2. threePrime: Coverage at 3' end of features; 3. bases:
#'   distance (in bp) to the boundary
#'
#' @examples
#' data(queryRegions)
#' data(gff)
#' txdb <- txdbmaker::makeTxDbFromGRanges(gff)
#' transcriptCoords <- GenomicFeatures::transcripts(txdb)
#' transcriptEndCoverageBin <- getFeatureBoundaryCoverageBin (
#'                                      queryRegions = queryRegions,
#'                                     featureCoords = transcriptCoords,
#'                                     flankSize = 100,
#'                                     sampleN = 1000)
#' @import GenomicRanges
#' @importFrom genomation ScoreMatrix
#' @export
getFeatureBoundaryCoverageBin <- function (queryRegions,
                                           featureCoords,
                                           flankSize = 50,
                                           sampleN = 0) {

  if (sampleN > 0 && sampleN < length(featureCoords)) {
    featureCoords <- sort(featureCoords[sample(length(featureCoords), sampleN)])
  }

  #flanking regions at/around 5' site of the features
  fivePrimeFlanks <- GenomicRanges::flank(x = featureCoords,
                                          width = flankSize,
                                          start = TRUE,
                                          both = TRUE
  )
  #flanking regions at/around 3' site of the features
  threePrimeFlanks <- GenomicRanges::flank(x = featureCoords,
                                           width = flankSize,
                                           start = FALSE,
                                           both = TRUE)

  cvgFivePrime <- genomation::ScoreMatrixBin(target = queryRegions,
                                             windows = fivePrimeFlanks,
                                             bin.num = 100,
                                             bin.op = 'max',
                                             strand.aware = TRUE)

  cvgThreePrime <- genomation::ScoreMatrixBin(target = queryRegions,
                                              windows = threePrimeFlanks,
                                              bin.num = 100,
                                              bin.op = 'max',
                                              strand.aware = TRUE)

  mdata <- data.frame('fivePrime' = colSums(cvgFivePrime),
                      'threePrime' = colSums(cvgThreePrime),
                      'bins' =  c(-50:-1, 1:50))

  return(mdata)
}


#' calculateCoverageProfile
#' 
#' This function checks overlaps between input query regions and annotation 
#' features, and then calculates coverage profile along target regions.
#' 
#' @param queryRegions GRanges object imported from a BED file using 
#'   \code{importBed} function
#' @param targetRegions GRanges object containing genomic coordinates of a 
#'   target feature (e.g. exons)
#' @param sampleN If set to a positive integer, \code{targetRegions} will be 
#'   downsampled to \code{sampleN} regions
#' @param bin.num Positive integer value (default: 100) to determine how many
#'   bins the targetRegions should be split into (See
#'   genomation::ScoreMatrixBin)
#' @param bin.op The operation to apply for each bin: 'min', 'max', or 'mean'
#'   (default: mean). (See genomation::ScoreMatrixBin)
#' @param strand.aware TRUE/FALSE (default: TRUE) The strands of target regions
#'   are considered.
#' @return A ScoreMatrix object returned by \code{genomation::ScoreMatrixBin} 
#'   function. Target regions are divided into 100 equal sized bins and coverage
#'   level is calculated in a strand-specific manner.
#' @examples
#' data(gff)
#' data(queryRegions)
#' txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)
#' df <- calculateCoverageProfile(queryRegions = queryRegions,
#'                               targetRegions = txdbFeatures$exons,
#'                                     sampleN = 1000)
#' @importFrom genomation ScoreMatrixBin
#' @import GenomicRanges
#' @export
calculateCoverageProfile = function (queryRegions, 
                                     targetRegions, 
                                     sampleN = 0, 
                                     bin.num = 100, 
                                     bin.op = 'mean', 
                                     strand.aware = TRUE){
  #remove windows shorter than bin.num
  #because the windows have to be separated into at least 'bin.num' bins
  windows <- targetRegions[GenomicRanges::width(targetRegions) >= bin.num]

  if (length(windows) > 0) {
    if (sampleN > 0 && sampleN < length(windows)) {
      windows <- sort(windows[sample(length(windows), sampleN)])
    }
    sm <- genomation::ScoreMatrixBin(target = queryRegions,
                                     windows = windows,
                                     bin.num = bin.num,
                                     bin.op = bin.op,
                                     strand.aware = strand.aware)
    return(sm)
    } else {
    stop("Cannot compute coverage profile for target regions.\n
         There are no target regions longer than",bin.num,"\n")
  }
}

#' calculateCoverageProfileList
#' 
#' This function checks overlaps between input query regions and a target list 
#' of annotation features, and then calculates the coverage profile along the 
#' target regions.
#' 
#' @param queryRegions GRanges object imported from a BED file using 
#'   \code{importBed} function
#' @param targetRegionsList A list of GRanges objects containing genomic 
#'   coordinates of target features (e.g. transcripts, exons, introns)
#' @param sampleN If set to a positive integer, \code{targetRegions} will be 
#'   downsampled to \code{sampleN} regions
#' @param bin.num Positive integer value (default: 100) to determine how many
#'   bins the targetRegions should be split into (See
#'   genomation::ScoreMatrixBin)
#' @param bin.op The operation to apply for each bin: 'min', 'max', or 'mean'
#'   (default: mean). (See genomation::ScoreMatrixBin)
#' @param strand.aware TRUE/FALSE (default: TRUE) The strands of target regions
#'   are considered.
#' @return A data.frame consisting of four columns: 1. bins level 2.
#'   meanCoverage 3. standardError 4. feature Target regions are divided into
#'   100 equal sized bins and coverage level is summarized in a strand-specific
#'   manner using the \code{genomation::ScoreMatrixBin} function. For each bin,
#'   mean coverage score and the standard error of the mean coverage score is
#'   calculated (\code{plotrix::std.error})
#' @importFrom plotrix std.error
#' @examples
#' data(gff)
#' data(queryRegions)
#' txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)
#' dfList <- calculateCoverageProfileList(queryRegions = queryRegions,
#'                               targetRegionsList = txdbFeatures,
#'                                     sampleN = 1000)
#' @export
calculateCoverageProfileList <- function (queryRegions,
                                          targetRegionsList,
                                          sampleN = 0, 
                                          bin.num = 100,
                                          bin.op = 'mean',
                                          strand.aware = TRUE) {
  results <- lapply(X = names(targetRegionsList),
                    FUN = function(x) { 
                      sm <- calculateCoverageProfile(queryRegions = queryRegions,
                                                     targetRegions = targetRegionsList[[x]],
                                                     sampleN = sampleN, 
                                                     bin.num = bin.num, 
                                                     bin.op = bin.op, 
                                                     strand.aware = strand.aware)
                      mdata <- data.frame('bins' = c(1:bin.num), 
                                          'meanCoverage' = colMeans(sm), 
                                          'standardError' = apply(sm, 2, plotrix::std.error),
                                          'feature' = x)
                    })
  return(do.call(rbind, results))
}

#' calculateCoverageProfileListFromTxdb
#'
#' This function is deprecated. Use ?calculateCoverageProfileList instead. 
#' 
#' @export
calculateCoverageProfileListFromTxdb <- function () {
  .Deprecated('calculateCoverageProfileList')
}

#' calculateCoverageProfileFromTxdb
#'
#' This function is deprecated. Use ?calculateCoverageProfile instead. 
#' 
#' @export
calculateCoverageProfileFromTxdb <- function () {
  .Deprecated('calculateCoverageProfile')
}

#' checkSeqDb
#' 
#' Given a string that denotes a genome version (e.g. hg19)
#' returns the BSgenome object matching the genome version 
#' that are available in BSgenome::available.genomes() 
#' 
#' @param genomeVersion String that denotes genome version. 
#' To unambigously select a BSgenome object, provide a string
#' that matches the end of the available genomes at: 
#' BSgenome::available.genomes().
#' @return Returns a BSgenome object that uniquely matches the 
#' genomeVersion. 
#' @examples 
#' checkSeqDb('hg19')
#' @importFrom BSgenome available.genomes
#' @export
checkSeqDb <- function(genomeVersion) {
  availableGenomes <- BSgenome::available.genomes()
  db <- grep(paste0(genomeVersion, '$'),
         BSgenome::available.genomes(),
         value = T)
  if (length(db) == 0) {
    stop(
      "Can't find a genome sequence with the given genome version:",
      genomeVersion,
      "\n\tRun BSgenome::available.genomes() to see which ones are available"
    )
  } else if (length(db) > 1) {
    stop(
      "Can't match the genome version to an unambigous genome sequence object.",
      "\nmatching genomes are: ",
      paste(db, collapse = '\t'),
      "\nPlease update your genomeVersion input to uniquely match",
      "\nAlso see BSgenome::available.genomes()"
    )
  }
  
  if (!requireNamespace(db, quietly = TRUE)) {
    stop(
      "Can't find the package ",
      db,
      " in your system. \n Please install via => ",
      paste0("BiocManager::install('", db, "')")
    )
  }
  
  message(date(), " => Returning ", db, " as BSgenome object")
  
  require(db, character.only = TRUE)
  return(get(db))
}

#' Generate a RCAS Report for a list of transcriptome-level segments
#' 
#' This is the main report generation function for RCAS. This function takes a
#' BED file, a GTF file to create a summary report regarding the annotation data
#' that overlap the input BED file, enrichment analysis for functional terms,
#' and motif analysis.
#' 
#' @param queryFilePath a BED format file which contains genomic coordinates of 
#'   protein-RNA binding sites
#' @param gffFilePath A GTF format file which contains genome annotations 
#'   (preferably from ENSEMBL)
#' @param annotationSummary TRUE/FALSE (default: TRUE) A switch to decide if 
#'   RCAS should provide annotation summaries from overlap operations
#' @param goAnalysis TRUE/FALSE (default: TRUE) A switch to decide if RCAS 
#'   should run GO term enrichment analysis
#' @param motifAnalysis TRUE/FALSE (default: TRUE) A switch to decide if RCAS 
#'   should run motif analysis
#' @param genomeVersion  A character string to denote for which genome version 
#'   the analysis is being done. 
#' @param outDir Path to the output directory. (default: current working 
#'   directory)
#' @param printProcessedTables boolean value (default: FALSE). If set to TRUE, 
#'   raw data tables that are used for plots/tables will be printed to text
#'   files.
#' @param sampleN integer value (default: 0). A parameter to determine if the 
#'   input query regions should be downsampled to a smaller size in order to 
#'   make report generation quicker. When set to 0, downsampling won't be done. 
#'   To activate the sampling a positive integer value that is smaller than the
#'   total number of query regions should be given.
#' @param quiet boolean value (default: FALSE). If set to TRUE, progress bars 
#' and chunk labels will be suppressed while knitting the Rmd file. 
#' @param selfContained boolean value (default: TRUE). By default, the generated
#' html file will be self-contained, which means that all figures and tables 
#' will be embedded in a single html file with no external dependencies 
#' (See rmarkdown::html_document) 
#' @return An html generated using rmarkdown/knitr/pandoc that contains 
#'   interactive figures, tables, and text that provide an overview of the 
#'   experiment
#' @examples
#' #Default run will generate a report using built-in test data for hg19 genome.
#' \dontrun{
#' runReport()
#' }
#' 
#' #A custom run for human
#' \dontrun{
#' runReport( queryFilePath = 'input.BED',
#'            gffFilePath = 'annotation.gtf', 
#'            genomeVersion = 'hg19')
#'            }
#' # To turn off certain modules of the report
#' \dontrun{
#' runReport( queryFilePath = 'input.BED',
#'            gffFilePath = 'annotation.gtf',
#'            motifAnalysis = FALSE,
#'            goAnalysis = FALSE )
#'            }
#' @import rmarkdown
#' @import knitr
#' @importFrom plotly plot_ly
#' @importFrom plotly add_trace
#' @importFrom plotly add_ribbons
#' @importFrom plotly layout
#' @importFrom seqLogo seqLogo
#' @import DT
#' @export
runReport <- function(queryFilePath = 'testdata',
                      gffFilePath = 'testdata',
                      annotationSummary = TRUE,
                      goAnalysis = TRUE,
                      motifAnalysis = TRUE,
                      genomeVersion = 'hg19',
                      outDir = getwd(),
                      printProcessedTables = FALSE,
                      sampleN = 0,
                      quiet = FALSE,
                      selfContained = TRUE) {

  db <- checkSeqDb(genomeVersion)
  # get species name 
  # this is needed for gprofiler functional enrichment 
  fields <- unlist(strsplit(db@metadata$organism, ' '))
  species <- tolower(paste0(unlist(strsplit(fields[1], ''))[1], 
                 fields[2]))
  
  if(queryFilePath != 'testdata') {
    queryFilePath <- normalizePath(queryFilePath)
  }

  if(gffFilePath != 'testdata') {
    gffFilePath <- normalizePath(gffFilePath)
  }
  
  reportFile <- system.file("reporting_scripts", "report.Rmd", package='RCAS')
  headerFile <- system.file("reporting_scripts", "header.html", package='RCAS')
  footerFile <- system.file("reporting_scripts", "footer.html", package='RCAS')
  
  outFile <- paste0(basename(queryFilePath), '.RCAS.report.html')

  rmarkdown::render(
    input = reportFile, 
    output_dir = outDir,
    intermediates_dir = outDir,
    output_file = outFile,
    output_format = rmarkdown::html_document(
      toc = TRUE,
      toc_float = TRUE,
      theme = 'simplex',
      number_sections = TRUE,
      includes = rmarkdown::includes(in_header = headerFile, 
                                     after_body = footerFile), 
      self_contained = selfContained
      ),
    params = list(query = queryFilePath,
                  gff = gffFilePath,
                  annotationSummary = annotationSummary,
                  goAnalysis = goAnalysis,
                  motifAnalysis = motifAnalysis,
                  genomeVersion = genomeVersion,
                  species = species,
                  printProcessedTables = printProcessedTables,
                  sampleN = sampleN,
                  workdir = outDir),
    quiet = quiet
    )
}

library(RCAS)
context("tests for main runReport function")

test_that("Testing runReport function", {
  expect_error(runReport(genomeVersion = 'foo'),
               regexp = "Can't find a genome sequence")
  expect_error(runReport(genomeVersion = '9'),
               regexp = "match the genome version to an unambigous genome sequence")
})

library(RCAS)
context("Functions to get txdbFeatures and calculating coverage profiles")

data(gff)

features <- getTxdbFeaturesFromGRanges(gffData = gff)
test_that("Parsing features from GFF", {
  expect_is(features, 'list')
  expect_equal(length(names(features)), 7)
  expect_match(names(features)[1], "transcripts")
  expect_match(names(features)[7], "threeUTRs")
})

data(queryRegions)
mytable <- getTargetedGenesTable(queryRegions = queryRegions,
                                 txdbFeatures = features)
test_that("Getting table of targeted genes", {
  expect_is(mytable, 'data.table')
  expect_equal(nrow(mytable), 2094)
  expect_equal(ncol(mytable), 8)
  expect_match(colnames(mytable)[1], "tx_name")
  expect_match(colnames(mytable)[8], "threeUTRs")
})

summary <- summarizeQueryRegions(queryRegions = queryRegions,
                                 txdbFeatures = features)
test_that("Summarizing query regions with txdbFeatures", {
  expect_is(summary, 'matrix')
  expect_match(colnames(summary), 'count')
  expect_equal(nrow(summary), length(names(features)) + 1)
})

profile <- calculateCoverageProfile(queryRegions = queryRegions,
                                   targetRegions = features[[1]],
                                   sampleN = 1000)
test_that("Getting coverage profile of query regions
          over a single gene feature from txdbFeatures", {
  expect_is(profile, 'ScoreMatrix')
  expect_equal(ncol(profile), 100)
})

profileList <- calculateCoverageProfileList(queryRegions = queryRegions,
                                            targetRegionsList = features,
                                            sampleN = 1000)
test_that("Getting a list of coverage profiles
          over all gene features from txdbFeatures", {
  expect_is(profileList, 'data.frame')
  expect_equal(colnames(profileList), c('bins', 'meanCoverage', 'standardError', 'feature'))
  expect_equal(dim(profileList), c(700, 4))
})


transcriptEndCoverage <- getFeatureBoundaryCoverage(
                                queryRegions = queryRegions,
                                featureCoords = features$transcripts[1:1000],
                                flankSize = 100, 
                                boundaryType = 'threeprime',
                                sampleN = 0
                                )

test_that("Calculating per base coverage profile at TES boundary", {
  expect_equal(colnames(transcriptEndCoverage), c("bases","meanCoverage","standardError"))
  expect_is(transcriptEndCoverage, 'data.frame')
  expect_equal(sum(transcriptEndCoverage$bases), -100)
})


transcriptEndCoverageBin <- getFeatureBoundaryCoverageBin(
  queryRegions = queryRegions,
  featureCoords = features$transcripts[1:1000],
  flankSize = 100,
  sampleN = 0
)

test_that("Calculating per bin coverage profile at TSS and TES boundaries", {
  expect_equal(colnames(transcriptEndCoverageBin), c("fivePrime","threePrime","bins"))
  expect_is(transcriptEndCoverageBin, 'data.frame')
  expect_equal(as.vector(colSums(transcriptEndCoverageBin)), c(15, 241, 0))
})

## functions to create, update and manipulate the sqlite database for meta-analysis 

#' @importFrom utils read.table
validateProjDataFile <- function(projDataFile, conn) {
  projData <- read.table(file = projDataFile, header = TRUE, 
                         stringsAsFactors = FALSE)
  if(sum(grepl(pattern = '-', x = projData$sampleName)) > 0){
    warning("Converting dashes (-) in sample names to underscore (_)\n")
    projData$sampleName <- gsub(pattern = '-', '_', x = projData$sampleName)
  }
  #check 1
  ambigousSamples <- projData[table(projData$sampleName) > 1,]$sampleName
  if(length(ambigousSamples) > 0) {
    stop("Some sample names are ambiguously assigned to multiple files in:\n",projDataFile,"\n",
         "See sample names:",paste0(ambigousSamples, collapse = ', '),"\n")
  }
  #check 2
  ambigousFiles <- projData[table(projData$bedFilePath) > 1,]$bedFilePath
  if(length(ambigousFiles) > 0) {
    stop("Some bed files are ambiguously assigned to multiple sample names",projDataFile,"\n",
         "See:",paste0(ambigousFiles, collapse = ', '),"\n")
  }
  
  #check 3
  unaccessibleFiles <- projData$bedFilePath[!sapply(projData$bedFilePath, 
                                                    file.exists)]
  if(length(unaccessibleFiles) > 0) {
    stop("Following files are not accessible or do not exist at the provided location:\n",
         "See:",paste0(unaccessibleFiles, collapse = ', '),"\n")
  }
  
  #check 4 (when an existing database is updated, samples in projData object that clash with the
  #existing sample names will be ignored)
  if(RSQLite::dbExistsTable(conn, 'processedSamples')) {
    ps <- RSQLite::dbReadTable(conn, 'processedSamples')
    existingSamples <- intersect(ps$sampleName, projData$sampleName)
    if(length(existingSamples) > 0) {
      warning("The following samples already exists in the database. 
              These samples will be ignored during the database update:",
              paste0(existingSamples, collapse = ','))
    }
    projData <- rbind(ps, projData[!projData$sampleName %in% existingSamples,])
  }
  return(projData)
}

insertTableGtfData <- function(conn, name, gtfFilePath) {
  if(RSQLite::dbExistsTable(conn, name)) {
    message("Reading GTF data from the database")
    gtfData <- GenomicRanges::makeGRangesFromDataFrame(
      df = RSQLite::dbReadTable(conn, name), keep.extra.columns = TRUE)
  } else {
    message("Importing GTF annotations")
    gtfData <- importGtf(filePath = gtfFilePath, 
                         readFromRds = FALSE, 
                         colnames = c('source', 'type', 'score', 
                                      'gene_id', 'gene_name', 
                                      'transcript_id', 'exon_id', 
                                      'exon_number'))
    RSQLite::dbWriteTable(conn = conn, name = name, 
                          value = data.table::as.data.table(gtfData))
    return(gtfData)
  }
}

insertTableBedData <- function(conn, projData) {
  if(RSQLite::dbExistsTable(conn = conn, name = 'bedData')) {
    message("Reading existing BED data from database")
    bedData <- GenomicRanges::makeGRangesListFromDataFrame(
      df = RSQLite::dbReadTable(conn, 'bedData'), 
      split.field = 'sampleName')
    
    #find which samples in projData don't have any data yet
    newSamples <- setdiff(projData$sampleName, names(bedData))
    if(length(newSamples) > 0) {
      message("Reading BED files for new samples")
      newBedData <- importBedFiles(
        filePaths = projData[projData$sampleName %in% newSamples,]$bedFilePath,
        colnames = c('chrom', 'start', 'end', 'strand'))
      names(newBedData) <- newSamples
      
      message("Appending new interval datasets in 'bedData' table")
      dt <- as.data.table(newBedData)
      colnames(dt)[2] <- 'sampleName'
      RSQLite::dbWriteTable(conn = conn, name = 'bedData', 
                            value = dt, 
                            append = TRUE)
      bedData <- c(bedData, newBedData)
    } 
    return(bedData)
    } else {
    bedData <- importBedFiles(
      filePaths = projData$bedFilePath,
      colnames = c('chrom', 'start', 'end', 'strand'))
    names(bedData) <- projData$sampleName
    message("Saving interval datasets in 'bedData' table")
    dt <- as.data.table(bedData)
    colnames(dt)[2] <- 'sampleName'
    RSQLite::dbWriteTable(conn = conn, name = 'bedData', 
                          value = dt, 
                          append = TRUE)
    return(bedData)
  } 
}

insertTableAnnotationSummaries <- function(conn, bedData, ...) {
  
  if(RSQLite::dbExistsTable(conn, 'annotationSummaries')) {
    annotationSummaries <- RSQLite::dbReadTable(conn, 'annotationSummaries')
    
    newSamples <- setdiff(names(bedData), annotationSummaries$sampleName)
    if(length(newSamples) > 0) {
      message("Calculating annotation summaries for new samples")
      newAnnot <- summarizeQueryRegionsMulti(queryRegionsList = bedData[newSamples], 
                                             ...)
      newAnnot <- data.table::as.data.table(newAnnot, keep.rownames = TRUE)
      colnames(newAnnot)[1] <- 'sampleName'
      
      message("Appending new annotation summaries in 'annotationSummaries' table")
      RSQLite::dbWriteTable(conn = conn, name = 'annotationSummaries', 
                            value = newAnnot, 
                            append = TRUE)
    } 
  } else {
    message("Calculating annotation summaries")
    #get feature overlap table
    annotationSummaries <- summarizeQueryRegionsMulti(queryRegionsList = bedData, 
                                                      ...)
    annotationSummaries <- data.table::as.data.table(x = annotationSummaries, 
                                                     keep.rownames = TRUE)
    colnames(annotationSummaries)[1] <- 'sampleName'
    message("Saving annotation summaries in 'annotationSummaries' table")
    RSQLite::dbWriteTable(conn = conn, name = 'annotationSummaries', 
                          value = annotationSummaries, 
                          append = TRUE)
    
  }
}

insertTableFeatureBoundaryCoverageProfiles <- function(conn, bedData, ...) {
  
  if(RSQLite::dbExistsTable(conn, 'featureBoundaryCoverageProfiles')) {
    cvg <- RSQLite::dbReadTable(conn, 'featureBoundaryCoverageProfiles')
    
    newSamples <- setdiff(names(bedData), cvg$sampleName)
    if(length(newSamples) > 0) {
      message("Calculating feature boundary coverage profiles for new samples")
      newCvg <- getFeatureBoundaryCoverageMulti(bedData[newSamples], ...)

      message("Appending new coverage profiles in 
              'featureBoundaryCoverageProfiles' table")
      RSQLite::dbWriteTable(conn = conn, 
                            name = 'featureBoundaryCoverageProfiles', 
                            value = newCvg, 
                            append = TRUE)
    } 
  } else {
    message("Calculating feature boundary coverage profiles")
    cvg <- getFeatureBoundaryCoverageMulti(bedData, ...)
    message("Saving coverage profiles in 
              'featureBoundaryCoverageProfiles' table")
    RSQLite::dbWriteTable(conn = conn, 
                          name = 'featureBoundaryCoverageProfiles', 
                          value = cvg, 
                          append = TRUE)
    
  }
}

insertTableDiscoveredMotifs <- function(conn, bedData, txdbFeatures, ...) {
  
  samples <- names(bedData)
  
  if(RSQLite::dbExistsTable(conn, 'discoveredMotifs')) {
    motifData <- RSQLite::dbReadTable(conn, 'discoveredMotifs')
    samples <- setdiff(names(bedData), motifData$sampleName)
  }
  
  if(length(samples) > 0) {
    message("Started motif discovery module for new samples...")
    newMotifData <- lapply(X = samples,
                           FUN = function(s) {
                             queryRegions <- bedData[[s]]
                             message("Discovering motifs for sample ",s)
                             results <- discoverFeatureSpecificMotifs(
                               queryRegions = queryRegions, 
                               txdbFeatures = txdbFeatures,  
                               ...)
                             results <- as.data.frame(do.call(rbind, lapply(names(results), function(x) {
                                 df <- getMotifSummaryTable(results[[x]])
                                 df$feature <- x
                                 return(df)
                               })))
                             results$sampleName <- s
                             return(results)
                           })
    newMotifData <- as.data.frame(do.call(rbind, newMotifData))
    
    message("Appending new motif discovery stats to 
            'discoveredMotifs' table")
    RSQLite::dbWriteTable(conn = conn, 
                          name = 'discoveredMotifs', 
                          value = newMotifData, 
                          append = TRUE)
  } else {
    message("All samples in the database already have motif analysis done")
  }
}

insertTableOverlapMatrix <- function(conn, name, bedData, ...) {
  if(RSQLite::dbExistsTable(conn, name)) {
    M <- RSQLite::dbReadTable(conn, name)
    
    newSamples <- setdiff(names(bedData), colnames(M))
    if(length(newSamples) > 0) {
      newM <- getIntervalOverlapMatrix(queryRegionsList = bedData[newSamples], ...)
      M <- cbind(data.table::as.data.table(M), 
                 data.table::as.data.table(newM))
    }
    RSQLite::dbWriteTable(conn = conn, name = name, 
                          value = M, 
                          overwrite = TRUE)
  } else {
      message("Running function: getIntervalOverlapMatrix for table",name)
      M <- getIntervalOverlapMatrix(queryRegionsList = bedData, ...)
      RSQLite::dbWriteTable(conn = conn, name = name, 
                            value = data.table::as.data.table(M, 
                                                              keep.rownames = TRUE), 
                            overwrite = TRUE)
  }
}

#' summarizeDatabaseContent 
#' 
#' Given a path to an sqlite database created using RCAS::createDB function,
#' accesses the database and provides a quick summary of available samples 
#' and number of entries of each sample in the available tables of the 
#' database.  
#' 
#' @param dbPath Path to the sqlite database
#' @return A data.frame object 
#' @import RSQLite
#' @export
summarizeDatabaseContent <- function(dbPath) {
  mydb <- RSQLite::dbConnect(RSQLite::SQLite(), dbPath)
  tables <- RSQLite::dbListTables(mydb)
  #exclude gtfData because it is independent from sample names
  tables <- tables[!tables %in% c('gtfData')]

  summary <- lapply(tables, function(tbl) {
    df <- RSQLite::dbReadTable(mydb, tbl)
    #tables that don't have 'sampleName' as a column have to be 
    #handled separately. 
    if (tbl == 'geneOverlaps') {
      r <- data.frame(apply(df[-1], 2, sum))
      colnames(r) <- tbl
      r$sampleName <- rownames(r)
      return(r)
    } else {
      r <- data.frame(table(df$sampleName))
      colnames(r) <- c('sampleName', tbl)
      return(r)
    }
  })
  summary <- Reduce(function(...) merge(..., all=T, by = 'sampleName'), summary)
  RSQLite::dbDisconnect(mydb)
  return(summary)
}

#' deleteSampleDataFromDB
#' 
#' Given a list of sample names, the function deletes all datasets calculated
#' for the given samples from the database.
#' 
#' @param dbPath Path to the sqlite database
#' @param sampleNames The names of the samples for which all relevant datasets
#'   should be deleted from the database. Tip: Use RSQLite::dbReadTable function
#'   to read the table 'processedSamples' to see which samples are available in
#'   the database.
#' @return SQLiteConnection object with updated contents in the dbPath 
#' @import RSQLite
#' @export
deleteSampleDataFromDB <- function(dbPath, sampleNames) {
  mydb <- RSQLite::dbConnect(RSQLite::SQLite(), dbPath)
  tables <- RSQLite::dbListTables(mydb)
  #exclude gtfData because it is independent from sample names
  tables <- tables[!tables %in% c('gtfData')]
  
  for(tbl in tables) {
    #tables that don't have 'sampleName' as a column have to be 
    #handled separately. tables with the column 'sampleName' 
    #can be updated all at the same time (see the else statement)
    if(tbl == 'geneOverlaps') {
      df <- RSQLite::dbReadTable(mydb, tbl)
      df <- df[,!colnames(df) %in% sampleNames]
      RSQLite::dbRemoveTable(mydb, 'geneOverlaps')
      RSQLite::dbWriteTable(mydb, 'geneOverlaps', df, append = TRUE)
      dfNew <- RSQLite::dbReadTable(mydb, 'geneOverlaps')
      message("Removing columns '",paste0(sampleNames, 
                                          collapse = ', '),
              "' from table ",tbl,
              "... Columns affected: ",ncol(dfNew)-ncol(df))
    } else {
      rs <- RSQLite::dbSendStatement(conn = mydb, 
                                     statement = paste0(
                                       'DELETE FROM ',tbl,
                                       ' WHERE "sampleName" = :s'))
      RSQLite::dbBind(rs, params = list(s = sampleNames))
      message("Removing rows '",paste0(sampleNames, 
                                       collapse = ', '),
              "' from table ",tbl,
              "... Rows affected: ",RSQLite::dbGetRowsAffected(rs))
      dbClearResult(rs)
    }
  }
  return(mydb)
}


#' createDB
#' 
#' Creates an sqlite database consisting of various tables of data obtained 
#' from processed BED files
#' 
#' @param dbPath Path to the sqlite database file (could be an existing file or 
#'   a new file path to be created at the given path)
#' @param projDataFile A file consisting of meta-data about the input samples. 
#'   Must minimally consist of two columns: 1. sampleName (name of the sample) 
#'   2. bedFilePath (full path to the location of the BED file containing data 
#'   for the sample)
#' @param gtfFilePath Path to the GTF file (preferably downloaded from the 
#'   Ensembl database) that contains genome annotations
#' @param update TRUE/FALSE (default: FALSE) whether an existing database 
#'   should be updated 
#' @param genomeVersion  A character string to denote for which genome version 
#'   the analysis is being done. Available options are hg19/hg38 (human), mm9/mm10 
#'   (mouse), ce10 (worm) and dm3 (fly).
#' @param annotationSummary TRUE/FALSE (default:TRUE) whether annotation summary
#' module should be run
#' @param coverageProfiles TRUE/FALSE (default: TRUE) whether coverage profiles
#' module should be run
#' @param motifAnalysis TRUE/FALSE (default: TRUE) whether motif discovery module
#' should be run 
#' @param nodeN Number of cpus to use for parallel processing (default: 1)
#' @return Path to an SQLiteConnection object created by RSQLite package
#' @importFrom RSQLite dbConnect
#' @importFrom RSQLite dbWriteTable
#' @importFrom RSQLite dbDisconnect
#' @importFrom RSQLite SQLite
#' @importFrom data.table as.data.table
#' @importFrom proxy dist
#' @examples 
#' FUS_path <- system.file("extdata", "FUS_Nakaya2013c_hg19.bed",
#' package='RCAS') 
#' 
#' FMR1_path <- system.file("extdata",
#' "FMR1_Ascano2012a_hg19.bed", package='RCAS') 
#' 
#' projData <- data.frame('sampleName' = c('FUS', 'FMR1'), 
#' 'bedFilePath' = c(FUS_path,FMR1_path), stringsAsFactors = FALSE)
#'  
#' write.table(projData, 'myProjDataFile.tsv', sep = '\t', quote =FALSE, 
#' row.names = FALSE) 
#' 
#' gtfFilePath <- system.file("extdata",
#' "hg19.sample.gtf", package='RCAS')
#' 
#' createDB(dbPath = 'hg19.RCASDB.sqlite', 
#' projDataFile = './myProjDataFile.tsv', 
#' gtfFilePath = gtfFilePath,  
#' genomeVersion = 'hg19', 
#' motifAnalysis = FALSE, 
#' coverageProfiles = FALSE)
#' 
#' #Note: to add new data to an existing database, set update = TRUE
#' 
#' @export
createDB <- function(dbPath = file.path(getwd(), 'rcasDB.sqlite'), 
                     projDataFile, gtfFilePath = '', update = FALSE, 
                     genomeVersion, 
                     annotationSummary = TRUE, 
                     coverageProfiles = TRUE, 
                     motifAnalysis = TRUE, 
                     nodeN = 1) {
  
  # check if seqdb is available for genome version
  seqDb <- checkSeqDb(genomeVersion)
  
  if(file.exists(dbPath) & update == FALSE) {
    stop("A database already exists at ",dbPath,"\n",
         "Either provide a different path or set argument 'update' to TRUE  
         in order to update this database")
  } else if(update == FALSE & !file.exists(gtfFilePath)) {
    stop("A GTF file doesn't exist at:\n",gtfFilePath,"\n")
  } else if(!file.exists(projDataFile)) {
    stop("A project data file doesn't exist at:",projDataFile,"\n")
  }
  #create connection to sqlite 
  mydb <- RSQLite::dbConnect(RSQLite::SQLite(), dbPath)
  #read and validate project data file 
  projData <- validateProjDataFile(projDataFile, mydb) 
  #import and save genome annotations
  gtfData <- insertTableGtfData(conn = mydb, name = 'gtfData',
                                gtfFilePath = gtfFilePath)
  message("Parsing transcript features")
  txdbFeatures <- getTxdbFeaturesFromGRanges(gtfData)
  
  bedData <- insertTableBedData(conn = mydb,projData = projData) 

  if(annotationSummary == TRUE) {
    insertTableAnnotationSummaries(conn = mydb, bedData = bedData, 
                                   txdbFeatures = txdbFeatures, nodeN = nodeN)
    
    geneRanges <- unlist(range(split(gtfData, gtfData$gene_name)))
    
    insertTableOverlapMatrix(conn = mydb, name = 'geneOverlaps',
                             bedData = bedData, 
                             targetRegions = geneRanges,
                             targetRegionNames = names(geneRanges),
                             nodeN = nodeN)
  }
  
  if(coverageProfiles == TRUE) {
    insertTableFeatureBoundaryCoverageProfiles(conn = mydb, bedData = bedData, 
                                             txdbFeatures = txdbFeatures, 
                                             sampleN = 10000)
  }
  
  if(motifAnalysis == TRUE) {
    insertTableDiscoveredMotifs(conn = mydb, 
                                bedData = bedData, 
                                txdbFeatures = txdbFeatures, 
                                genomeVersion = genomeVersion, 
                                motifN = 1,
                                nCores = 1,
                                resizeN = 15, 
                                sampleN = 10000)
  }
  
  RSQLite::dbWriteTable(mydb, 'processedSamples', projData, overwrite = TRUE)
  RSQLite::dbDisconnect(mydb)
  message("Finished preparing the database and disconnected.",
      "\nType",paste0(" RSQLite::dbConnect(RSQLite::SQLite(), \'",
                    dbPath,"\')")," to reconnect")
  return(dbPath)
} 

library(RCAS)
context("tests for setting up and modifying sqlite database")

dbPath <- system.file("extdata", "hg19.RCASDB.sqlite", package='RCAS')
dbPathTest <- file.path(getwd(), 'test.RCASDB.sqlite')
file.copy(from = dbPath, to = dbPathTest, overwrite = TRUE)

# Testing createDB function
test_that("Testing createDB function", {
  expect_error(createDB(dbPath = dbPath,
                        genomeVersion = 'hg19',
                        update = FALSE),
               regexp = 'A database already exists')
})

## Testing summarizeDatabaseContent function
summary <- summarizeDatabaseContent(dbPath = dbPathTest)
test_that("Testing summarizeDatabaseContent function", {
  expect_is(summary, 'data.frame')
  expect_equal(nrow(summary), 2)
  expect_equal(ncol(summary), 5)
  expect_identical(as.vector(summary$sampleName), c("FMR1", "FUS"))
})

mydb <- deleteSampleDataFromDB(dbPath = dbPathTest, sampleNames = 'FUS')
samples <- RSQLite::dbReadTable(mydb, 'processedSamples')

## Testing deleteSampleDataFromDB function
test_that("Testing deleteSampleDataFromDB function", {
  expect_is(mydb, "SQLiteConnection")
  expect_false('FUS' %in% samples$sampleName)
  expect_true('FMR1' %in% samples$sampleName)
})

RSQLite::dbDisconnect(mydb)

if(file.exists(dbPathTest)) {
  file.remove(dbPathTest)
}

library(RCAS)
context("built-in data: gff, queryRegions")

data(gff)
test_that("gff is a GRanges object containing test data of dimensions", {
  expect_is(object = gff, 'GRanges')
  expect_equal(length(gff), 238010)
  expect_equal(ncol(GenomicRanges::mcols(gff)), 16)
})

data(queryRegions)
test_that("queryRegions is a GRanges object from a HITS-CLIP dataset", {
  expect_is(object = queryRegions, 'GRanges')
  expect_equal(length(queryRegions), 10000)
  expect_equal(round(mean(GenomicRanges::score(queryRegions))), 45)
})

---
title: "How to do meta-analysis of CLIP-seq peaks from multiple samples with RCAS"
author: "Bora Uyar, Ricardo Wurmus, Altuna Akalin"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{How to do meta-analysis of multiple samples}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r global_options, include=FALSE}
knitr::opts_chunk$set(warning = FALSE, message = FALSE, eval = FALSE, fig.width = 7, fig.height = 4.2)
```


```{r load_libraries, results='hide'}
library(RCAS)
```

**Warning**: Due to space limitations, we demonstrate the useful functions in this workflow without images. In order to see an HTML report generated based on the full genome annotations and complete datasets described in the vignette, please see [here](https://bimsbstatic.mdc-berlin.de/akalin/buyar/RCAS/1.4.2/RCAS.html).

For the most up-to-date functionality, usage and installation instructions, and example outputs, see our [github repository here](https://github.com/BIMSBbioinfo/RCAS). 

# Introduction

The first release of RCAS was designed to produce quality controls and exploratory plots for the analysis of transcriptome-wide regions of interest with regard to their feature specific distributions, sequence motif signals, and biological function enrichments. However, this workflow didn't include functions to directly compare multiple samples. However, it may be often desirable to see how the biological content of one sample relates to the content obtained from different samples. For instance, one can design a CLIP-seq based experiment with multiple conditions each containing multiple biological replicates and want to address the following common questions: 
  
   - Are the transcript-feature specific distributions of the samples **reproducible across biological replicates**? 
   - Can we observe differences between the motif patterns of samples from different conditions? If so, can we pinpoint which transcript features shows the biggest difference? 
   
Since the release of RCAS 1.4.0, it is possible to process more than one input BED files and generate a self-containing HTML report with plots and tables that compare/contrast the input samples. 

In this vignette, we will use example datasets from CLIP based sequencing experiments to show how to use the meta-analysis functionality of the RCAS package. However, the same workflow can be applied to any kind of high-throughput dataset that pertains to transripts and can be expressed as genomic ranges in a BED format file. 

# Preparing the inputs

When starting from scratch, the two necessary inputs are:
  
  1. **Project Data File**: A tab-separated file that contains information about the samples and BED file locations of each sample. This file must contain minimally two columns:
    - sampleName: The name of the sample (must be unique in each row)
    - bedFilePath: The path to the BED format file that contains the genomic ranges for the analysed sample. 
  2. **GTF File**: GTF file contains the genome annotations in [GTF format](https://www.ensembl.org/info/website/upload/gff.html). 
  
## Project Data File

In this vignette, we will analyse the peak regions discovered via CLIP-sequencing experiments of the [RNA-binding protein FUS](http://www.uniprot.org/uniprot/P35637) by [Nakaya et al, 2013](https://www.ncbi.nlm.nih.gov/pubmed/23389473), [Synaptic Functional Regulator FMR1](http://www.uniprot.org/uniprot/Q06787) by [Ascano et al. 2012](https://www.ncbi.nlm.nih.gov/pubmed/23235829), and [Eukaryotic initiation factor 4A-III](http://www.uniprot.org/uniprot/P38919) by [Sauliere et al, 2012](https://www.ncbi.nlm.nih.gov/pubmed/23085716). We use two replicates from both experiments and constrain the analysis to a small portion of the genome (the first 1 million base pairs of chromosome 1). Both the genome annotations and the peaks detected within these regions are available as built-in data in the RCAS package. To obtain the full datasets, please refer to the [doRiNa database](http://dorina.mdc-berlin.de/regulators). 

First, we detect the paths to these datasets:
```{r}
FUS_rep1_path <- system.file('extdata', 'FUS_Nakaya2013c_hg19.bed', package = 'RCAS')
FUS_rep2_path <- system.file('extdata', 'FUS_Nakaya2013d_hg19.bed', package = 'RCAS')
FMR1_rep1_path <- system.file('extdata', 'FMR1_Ascano2012a_hg19.bed', package = 'RCAS')
FMR1_rep2_path <- system.file('extdata', 'FMR1_Ascano2012b_hg19.bed', package = 'RCAS')
EIF4A3_rep1_path <- system.file('extdata', 'EIF4A3Sauliere20121a.bed', package = 'RCAS')
EIF4A3_rep2_path <- system.file('extdata', 'EIF4A3Sauliere20121b.bed', package = 'RCAS')
```

Then, we create a file containing the sample names and the locations of the BED files for each sample. 
```{r}
projData <- data.frame('sampleName' = c('FUS_1', 'FUS_2', 'FMR1_1', 'FMR1_2', 'EIF4A3_1', 'EIF4A3_2'), 
                       'bedFilePath' = c(FUS_rep1_path, FUS_rep2_path, 
                                         FMR1_rep1_path, FMR1_rep2_path, 
                                         EIF4A3_rep1_path, EIF4A3_rep2_path), 
                       stringsAsFactors = FALSE)

projDataFile <- file.path(getwd(), 'myProjDataFile.tsv') 
write.table(projData, projDataFile, sep = '\t', quote =FALSE, row.names = FALSE) 
```

## GTF File (genome annotations)

The second input is the path to the GTF file containing genome annotations. 
Here we use the first 1 million bases of the chromosome 1 from the Ensembl Database (version 75 - corresponds to GRCh37 build - hg19 in UCSC). 

```{r}
gtfFilePath <- system.file("extdata", "hg19.sample.gtf", package = "RCAS")
```

# Creating a RSQLite database  

In order to avoid redundant preprocessing of the same input files, we have developed a function `RCAS::createDB` to save all preprocessed data into an SQLite database using the R package [RSQLite](https://cran.r-project.org/web/packages/RSQLite/index.html). With this function, all input files from the project data file are processed, the resulting processed data are saved in the following SQL tables: 
 
  - **gtfData**: Contains the gene annotations parsed from the `gtfFilePath` argument. The fields of this table are: seqnames, start, end, width, strand, source, type, score, gene_id, gene_name, transcript_id, exon_id, and exon_number. 
  - **bedData**: Contains all the content of the input BED files. The fields of the table are: group, sampleName, seqnames, start, end, width, and strand.    
  - **annotationSummaries**: Contains the number of overlaps between the input sample and the target transcript features categorised by feature type (i.e. transcripts, exons, promoters, UTRs, introns, cds). 
  - **geneOverlaps**: Contains the number of overlaps between the input sample and the target genes. 
  - **featureBoundaryCoverageProfiles**: Contains the coverage scores computed by [genomation::scoreMatrix function](https://bioconductor.org/packages/release/bioc/vignettes/genomation/inst/doc/GenomationManual.html)
  - **discoveredMotifs**: Contains enriched motifs per each transcript feature for each sample.  
  - **processedSamples**: Contains a mapping between the processed sample and the path to the BED file that contains the data for the corresponding sample. 

To create a database with preprocessed data, use `RCAS::createDB()` function. When starting a fresh database, we can provide a file name that does not exist at the corresponding folder must be provided. If a database already exists at the given location, an error will be returned. 
```{r}
databasePath <- file.path(getwd(), 'myProject.sqlite')
invisible(createDB(dbPath = databasePath, projDataFile = projDataFile, gtfFilePath = gtfFilePath, genomeVersion = 'hg19'))
```

However, it is also possible to update an existing database by setting the `update` argument to `TRUE`. This is useful when you want to add new datasets to an existing project. This will only add processed data for samples that don't already exist in the database. This will not attempt to overwrite existing data for existing samples. 

```
createDB(dbPath = databasePath, projDataFile = projDataFile, gtfFilePath = gtfFilePath, genomeVersion = 'hg19', update = TRUE)
```

It is also possible to turn off certain analysis modules by setting `annotationSummary`, `coverageProfiles`, or `motifAnalysis` modules to `FALSE`. For instance, the following command will create the same database except for the `discoveredMotifs` table.

```
createDB(dbPath = databasePath, projDataFile = projDataFile, gtfFilePath = gtfFilePath, genomeVersion = 'hg19', motifAnalysis = FALSE)
```

If the user wishes to overwrite datasets for existing samples, all relevant data for the given samples must be first purged from the database. It is possible to do so via:

Here we erase all entries relevant to samples 'FMR1_1' and 'FMR1_2'. 
```{r}
RCAS::deleteSampleDataFromDB(dbPath = databasePath, sampleNames = c('FMR1_1', 'FMR1_2'))
```
 
To have a quick look into the contents of any given database, we can use the `RCAS::summarizeDatabaseContent()` function. 
This returns a table of number of row entries for each sample in any given table that exists in the database. 
```{r}
knitr::kable(RCAS::summarizeDatabaseContent(dbPath = databasePath))
```

Each table of the database can be read into memory using the `RSQLite::dbReadTable()` function. 
To read a specific table from a database, firstly a connection to the sqlite dump needs to be created. 
```{r}
mydb <- RSQLite::dbConnect(RSQLite::SQLite(), databasePath)
```

List the tables that exist in the connection:
```{r}
RSQLite::dbListTables(mydb)
```

A table can be read into an R object:
```{r}
annotationSummaries <- RSQLite::dbReadTable(mydb, 'annotationSummaries')
knitr::kable(annotationSummaries)
```

**Warning**: It is important to consider that the tables read into an R object from an sqlite database are of the `data.frame` class. 
For most tables it is the desired format, however some of the tables need converting from `data.frame` to the desired class (e.g. `GenomicRanges` for `gtfData` table) for downstream processing. 


# Generating a meta-analysis report 

Once an sqlite database is obtained, `RCAS::runReportMetaAnalysis()` function can be utilized to quickly generate comparative analysis reports between whichever sample groups are desired to be compared. To generate this report, only two inputs are required:

  1. **dbPath**: The path to a pre-generated RCAS sqlite database.
  2. **sampleTablePath**: A tab-separated file with two columns (no rownames) 
    - header 1: sampleName. Name of the sample to include in the report. This must exist in the database. 
    - header 2: sampleGroup. This can be anything that somehow logically groups the entries in the `sampleName` column. For example, replicates of the same biological condition can be assigned the same `sampleGroup` value. 
  
Here is how to generate a stand-alone HTML report with interactive figures and tables from a pre-calculated RCAS database that compares CLIP datasets from two replicates of `FUS` and two replicates of `EIF4A3`. 

Let's first create a sample data file. The sample names found in this file should be a subset of the original project data file used to generate the sqlite database. 

```{r}
sampleData <- data.frame('sampleName' = c('FUS_1', 'FUS_2', 'EIF4A3_1', 'EIF4A3_2'), 
                         'sampleGroup' = c('FUS', 'FUS', 'EIF4A3', 'EIF4A3'), 
                         stringsAsFactors = FALSE) 
sampleDataFile <- file.path(getwd(), 'mySampleDataTable.tsv')
write.table(sampleData, sampleDataFile, sep = '\t', quote =FALSE, row.names = FALSE)
```

Now, we are ready to get an HTML report:
```{r}
runReportMetaAnalysis(dbPath = databasePath, sampleTablePath = sampleDataFile, 
                      outFile = file.path(getwd(), 'myProject.html'))
```


# Acknowledgements

RCAS is developed in the group of 
[Altuna Akalin](http://bioinformatics.mdc-berlin.de/team.html#altuna-akalin-phd)
(head of the Scientific Bioinformatics Platform) at the Berlin Institute of Medical Systems Biology
([BIMSB](https://www.mdc-berlin.de/13800178/en/bimsb))
at the Max-Delbrueck-Center for Molecular Medicine
([MDC](https://www.mdc-berlin.de)) in Berlin.

RCAS is developed as a bioinformatics service as part of
the [RNA Bioinformatics Center](http://www.denbi.de/index.php/rbc),
which is one of the eight centers of
the German Network for Bioinformatics Infrastructure
([de.NBI](http://www.denbi.de/)).  

To cite RCAS in publications use:

  > Bora Uyar, Dilmurat Yusuf, Ricardo Wurmus, Nikolaus Rajewsky, Uwe Ohler, Altuna Akalin; RCAS: an RNA centric annotation system
  > for transcriptome-wide regions of interest. Nucleic Acids Res 2017 gkx120. doi: 10.1093/nar/gkx120

---
title: "The RNA Centric Analysis System Report"
author: "Bora Uyar, Ricardo Wurmus, Altuna Akalin"
date: '`r Sys.time()`'
params:
  dbPath: './test.RCASdb.sqlite'
  sampleTablePath: ''
  workdir: '.'
---

```{r global_options, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, warning = FALSE, message = FALSE, fig.height = 4.5, fig.width = 8)
knitr::opts_knit$set(progress = FALSE, root.dir = params$workdir)
```

```{r setup}
suppressWarnings(suppressMessages(library(RCAS)))
# Connect to sqlite database
mydb <- RSQLite::dbConnect(RSQLite::SQLite(), params$dbPath)

# Read sample table and check format
sampleTable <- read.table(params$sampleTablePath, header = TRUE,stringsAsFactors = FALSE)
if(sum(c('sampleName', 'sampleGroup') %in% colnames(sampleTable)) != 2) {
  stop("The sample table must consist of at least 
       two columns (separated by white space(s)) 
       with headers 'sampleName' and 'sampleGroup'. 
       See ",params$sampleTable)
}

if(nrow(sampleTable) < 2) {
  stop("The sample table must contain at least one entry. 
       See ",sampleTable)
}

#set up which chunks to run depending on which tables are available in mydb
#1. geneOverlaps table
evalGeneOverlaps <- FALSE
if(RSQLite::dbExistsTable(mydb, 'geneOverlaps')){
  geneOverlaps <- RSQLite::dbReadTable(mydb, 'geneOverlaps')
  samplesNotFound <- setdiff(sampleTable$sampleName, colnames(geneOverlaps))
  if(length(samplesNotFound) > 0){
    warning("Samples with names '",paste0(samplesNotFound, collapse = ', '),
            "' do not exist in 'geneOverlaps' table. Skipping this table")
    } else {
      evalGeneOverlaps <- TRUE
      geneOverlaps <- subset(geneOverlaps, select = sampleTable$sampleName)
    }
  } 

#2. annotationSummaries table
evalAnnotationSummaries <- FALSE
if(RSQLite::dbExistsTable(mydb, 'annotationSummaries')){
  AS <-  RSQLite::dbReadTable(mydb, 'annotationSummaries')
  annotationSummaries <- as.matrix(AS[,c(-1)])
  rownames(annotationSummaries) <- AS$sampleName
  samplesNotFound <- setdiff(sampleTable$sampleName, rownames(annotationSummaries))
  
  if(length(samplesNotFound) > 0) {
    warning("Samples with names '",paste0(samplesNotFound, collapse = ', '),
          "' do not exist in 'annotationSummaries' table. Skipping this table")
    } else {
      evalAnnotationSummaries <- TRUE
      annotationSummaries <- annotationSummaries[sampleTable$sampleName,]
    }
} 

#3. discoveredMotifs table 
evalMotifDiscovery <- FALSE
if(RSQLite::dbExistsTable(mydb, 'discoveredMotifs')) {
  motifData <- RSQLite::dbReadTable(mydb, 'discoveredMotifs')
  samplesNotFound <- setdiff(sampleTable$sampleName, motifData$sampleName)
  if(length(samplesNotFound) > 0) {
    warning("Samples with names '",paste0(samplesNotFound, collapse = ', '),
            "' do not exist in 'discoveredMotifs' table. Skipping this table")
    } else {
      evalMotifDiscovery <- TRUE
      motifData <- motifData[motifData$sampleName %in% sampleTable$sampleName,]
    }
} 

#4. featureBoundaryCoverageProfiles table
evalCoverageProfiles <- FALSE 
if(RSQLite::dbExistsTable(mydb, 'featureBoundaryCoverageProfiles')) {
  cvg <-  RSQLite::dbReadTable(mydb, 'featureBoundaryCoverageProfiles')
  samplesNotFound <- setdiff(sampleTable$sampleName, cvg$sampleName)
  if(length(samplesNotFound) > 0) {
    warning("Samples with names '",paste0(samplesNotFound, collapse = ', '),
            "' do not exist in 'featureBoundaryCoverageProfiles' table. Skipping this table")
    } else {
      evalCoverageProfiles <- TRUE
      cvg <- cvg[cvg$sampleName %in% sampleTable$sampleName,]
      cvg$sampleGroup <- sampleTable[match(cvg$sampleName, sampleTable$sampleName),]$sampleGroup
    }
}

#initialize figure and table counts for captions
figureCount <- 1
tableCount <- 1
```


```{r geneOverlapsHeader, results='asis', eval=evalGeneOverlaps}
cat("# Distance of samples by various metrics\n")
```


```{r geneOverlapsFigureCaption, results='asis', eval=evalGeneOverlaps}
cat('## Jaccard distance based on shared target genes\n')

cat("**Figure",figureCount,":** Jaccard distance based on shared target genes. 
Here we plot the distance matrix between samples based on the Jaccard index computed for each pairwise sample 
based on how many genes the compared pair of samples co-overlap. \n")
```

```{r geneOverlaps, eval = evalGeneOverlaps}
SM <- as.matrix(proxy::dist(x = as.matrix(geneOverlaps), method = 'Jaccard', by_rows = FALSE, diag = TRUE, upper = TRUE))
pheatmap::pheatmap(SM, display_numbers = FALSE)
figureCount <- figureCount + 1
```

```{r annotationSummariesFigureCaption, results = 'asis', eval = evalAnnotationSummaries}
cat("## Clustering of samples based on overlaps with transcript features\n")
cat("**Figure",figureCount,":** Clustering of samples based on overlaps with transcript features\n")
```

```{r annotationSummaries, eval = evalAnnotationSummaries}
annDF <- data.frame('sampleGroup' = sampleTable[match(rownames(annotationSummaries), sampleTable$sampleName),]$sampleGroup)
rownames(annDF) <- rownames(annotationSummaries)
pheatmap::pheatmap(t(annotationSummaries), scale = 'column', 
         annotation_col = annDF)
figureCount <- figureCount + 1
```

```{r motifDiscoveryHeader, results = 'asis', eval = evalMotifDiscovery}
cat("# Motif Discovery\n")
cat("**Table",tableCount,":** Feature specific motifs discovered for each sample\n")
```

```{r motifDiscovery, eval = evalMotifDiscovery}
DT::datatable(motifData, 
          extensions = c('Buttons', 'FixedColumns'), 
          options = list(fixedColumns = TRUE, 
                         scrollX = TRUE,
                         dom = 'Bfrtip',
                         buttons = c('copy', 'print', 'csv','excel', 'pdf')),
          filter = 'bottom'
          )
tableCount <- tableCount + 1
```

```{r coverageProfilesHeader, results = 'asis', eval = evalCoverageProfiles}
cat("# Feature Boundary Coverage Profiles\n")
```

```{r coverageProfilesLinePlotFigureCaption, results = 'asis', eval = evalCoverageProfiles}
cat("## Smoothed line plots\n")
cat("**Figure",figureCount,":** Feature boundary coverage profiles for each sample\n")
```

```{r coverage_profiles, eval = evalCoverageProfiles, fig.width=8, fig.height=10}
#scale meanCoverage values by sampleName and feature
cvgScaled <- do.call(rbind, 
                     lapply(split(cvg, cvg$sampleName),
                            FUN = function(df) {
                              df$scaledCoverage <- scale(df$meanCoverage)
                              return(df)
                              }))

p <- plotly::ggplotly(ggplot2::ggplot(cvgScaled, aes(x = bases, y = scaledCoverage)) + 
                        geom_vline(xintercept = 0, color = 'gray') +
                        facet_grid(feature ~ boundary) + 
                        geom_smooth(aes(color = sampleGroup, alpha = scaledCoverage)) + theme_minimal())
layout(p)
figureCount <- figureCount + 1
```


```{r pca_coverageProfiles, eval = evalCoverageProfiles}
plotPCA <- function(cv, title = '') {
  cvM <- dcast(cv, sampleName ~ boundary + bases, value.var = 'meanCoverage')
  M <- as.matrix(cvM[,-1])
  rownames(M) <- cvM[,1]
  pca1 <- prcomp(M)
  scores = as.data.frame(pca1$x)
  scores$sampleName <- cv[match(rownames(scores), cv$sampleName),]$sampleName
  scores$sampleGroup <- cv[match(rownames(scores), cv$sampleName),]$sampleGroup

  p <- ggplot2::ggplot(data = scores, aes(x = PC1, y = PC2, label = sampleName)) +
    geom_point(aes(color = sampleGroup), size = 4) +
    geom_hline(yintercept = 0, colour = "gray65") +
    geom_vline(xintercept = 0, colour = "gray65") +
    labs(title = title) + theme_minimal(base_size = 18) %+replace% 
    theme(legend.position = 'bottom', legend.title = element_blank())
  return(p)
}

pcaPlots <- lapply(unique(cvg$feature), function(f) {
  plotPCA(cv = cvg[cvg$feature == f,], title = f)
})
names(pcaPlots) <- unique(cvg$feature)
```

```{r pca_coverageProfilesPlot, results='asis', echo = FALSE, eval = evalCoverageProfiles}
cat("## PCA plots {.tabset}\n")
for (i in 1:length(pcaPlots)) {
  cat("### ",names(pcaPlots)[i],"\n")
  print(pcaPlots[[i]])
  cat('\n\n')
}
figureCount <- figureCount + 1
```

```{r exit}
RSQLite::dbDisconnect(mydb)
```

# Acknowledgements

RCAS is developed in the group of 
[Altuna Akalin](http://bioinformatics.mdc-berlin.de/team.html#altuna-akalin-phd)
(head of the Scientific Bioinformatics Platform) by
[Bora Uyar](http://bioinformatics.mdc-berlin.de/team.html#bora-uyar-phd)
(Bioinformatics Scientist),
[Dilmurat Yusuf](http://bioinformatics.mdc-berlin.de/team.html#dilmurat-yusuf-phd)
(Bioinformatics Scientist) and 
[Ricardo Wurmus](http://bioinformatics.mdc-berlin.de/team.html#ricardo-wurmus)
(System Administrator) at the Berlin Institute of Medical Systems Biology
([BIMSB](https://www.mdc-berlin.de/13800178/en/bimsb))
at the Max-Delbrueck-Center for Molecular Medicine
([MDC](https://www.mdc-berlin.de)) in Berlin.

RCAS is developed as a bioinformatics service as part of
the [RNA Bioinformatics Center](http://www.denbi.de/index.php/rbc),
which is one of the eight centers of
the German Network for Bioinformatics Infrastructure
([de.NBI](http://www.denbi.de/)).  

# Session Information
```{r sessionInfo}
sessionInfo()
```

---
title: "The RNA Centric Analysis System Report"
author: "Bora Uyar, Dilmurat Yusuf, Ricardo Wurmus, Altuna Akalin"
date: "`r Sys.time()`"
params:
  query: 'testdata'
  gff: 'testdata'
  annotationSummary: TRUE
  goAnalysis: TRUE
  motifAnalysis: TRUE
  genomeVersion: 'hg19'
  species: 'hsapiens'
  printProcessedTables: FALSE
  sampleN: 0
  workdir: '.'
---

```{r global_options, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, warning = FALSE, message = FALSE, fig.height = 4.5, fig.width = 8)
knitr::opts_knit$set(progress = FALSE, root.dir = params$workdir)
```

```{r chunk_eval_options}
annotationSummary <- params$annotationSummary
goAnalysis <- params$goAnalysis
motifAnalysis <- params$motifAnalysis
provideMotifSummary <- FALSE #if params$motifAnalysis is TRUE and any motif is found, this will set to TRUE
goSummary <- FALSE 
```

```{r load_libraries, results='hide'}
library(RCAS)
```

# Introduction

RCAS is an automated system that provides dynamic genome annotations for custom
input files that contain transcriptomic regions. Such transcriptomic regions
could be, for instance, peak regions detected by CLIP-Seq analysis that detect
protein-RNA interactions, RNA modifications (alias the epitranscriptome),
CAGE-tag locations, or any other collection of target regions at the level of
the transcriptome.

RCAS is designed as a reporting tool for the functional analysis of RNA-binding
sites detected by high-throughput experiments. It takes as input a BED format
file containing the genomic coordinates of the RNA binding sites and a GTF file
that contains the genomic annotation features usually provided by publicly
available databases such as Ensembl and UCSC. RCAS performs overlap operations
between the genomic coordinates of the RNA binding sites and the genomic
annotation features and produces in-depth annotation summaries such as the
distribution of binding sites with respect to gene features (exons, introns,
5’/3’ UTR regions, exon-intron boundaries, promoter regions, and whole
transcripts) along with functionally enriched term for targeted genes. Moreover,
RCAS can darry out discriminative motif discovery in the query regions. The
final report of RCAS consists of high-quality dynamic figures and tables, which
are readily applicable for publications or other academic usage.

```{r initiate_figure_table_counts}
figureCount <- 1
tableCount <- 1
```

# Input Settings
```{r printInputSettings}
inputParameterDesc <- c('Query BED file', 
                     'Target GTF file',
                     'Annotation Summary Module',
                     'GO Analysis Module',
                     'Motif Analysis Module',
                     'Genome Version',
                     'Species',
                     'Print Processed Tables?',
                     'Randomly sample query regions down to (N)',
                     'Working Directory')
inputParameterValues <- c(params$query, 
                          params$gff,
                          params$annotationSummary,
                          params$goAnalysis,
                          params$motifAnalysis,
                          params$genomeVersion,
                          params$species,
                          params$printProcessedTables,
                          params$sampleN,
                          params$workdir)

inputSettings <- data.frame(parameters = inputParameterDesc,
                            values = inputParameterValues, 
                            stringsAsFactors = FALSE)

DT::datatable(data = inputSettings,
              extensions = 'FixedColumns',
              options = list(fixedColumns = TRUE, 
                         scrollX = TRUE, 
                         pageLength = 20,
                         dom = 't'))
```


```{r getInput, results='hide'}
if(params$query == 'testdata') {
  data(queryRegions) 
} else {
  cat('importing BED from filepath',params$query,'\n')
  queryRegions <- importBed(filePath = params$query, sampleN = params$sampleN)
}

if(params$gff == 'testdata') {
  data(gff)         
} else {
  cat('importing GTF from filepath',params$gff,'\n')
  gff <- importGtf(filePath = params$gff)
}

overlaps <- queryGff(queryRegions = queryRegions, gff = gff)
#data.table is used to do quick summary operations
overlaps.dt <- data.table(GenomicRanges::as.data.frame(overlaps)) 

#get all genes from the GTF data 
backgroundGenes <- unique(gff$gene_id)
#get genes that overlap query regions
targetedGenes <- unique(overlaps$gene_id)

```

```{r getTxdbFeatures}
txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)
```

```{r summaryOverlapsHeader, results='asis', eval=annotationSummary}
cat('# Annotation Summary for Query Regions \n')
```

```{r summarizeQueryRegionsFigureCaption, results='asis', eval=annotationSummary}
cat('## Distribution of query regions across gene features\n')

cat("**Figure",figureCount,":** The number of query regions that overlap different kinds of gene features are counted. The 'y' axis denotes the types of gene features included in the analysis and the 'x' axis denotes the percentage of query regions (out of total number of query regions denoted with 'n') that overlap at least one genomic interval that host the corresponding feature. Notice that the sum of the percentage values for different features don't add up to 100%, because some query regions may overlap multiple kinds of features \n")
```


```{r summarizeQueryRegions, eval=annotationSummary}
summary <- summarizeQueryRegions(queryRegions = queryRegions, 
                                 txdbFeatures = txdbFeatures)

df <- data.frame(summary)
df$percent <- round((df$count / length(queryRegions)), 3) * 100
p <- plotly::plot_ly( data = df, 
              x = rownames(df), 
              y = df$percent, 
              type = 'bar',
              text = paste("count:", df$count), 
              color = rownames(df)
              )
plotly::layout(p = p, 
       xaxis = list(title = 'features'),
       yaxis = list(title = paste("% query,", 
                                  "n =", length(queryRegions)
                                  )
                    ), 
       margin = list(b = 150, r = 50),
       font = list(size = 14)
       )

if(params$printProcessedTables == TRUE) {
  write.table(x = df, file=paste0('Figure',figureCount,'.summarizeQueryRegions.data.tsv'), quote = FALSE, sep = '\t', row.names = TRUE)
} 

figureCount <- figureCount + 1
```

```{r get_table_of_genes_tabcap, echo=FALSE, results='asis', eval=annotationSummary}
cat("## Interactive table of genes that overlap query regions\n")
cat("**Table",tableCount,":** Interactive table of top 100 genes that overlap query regions, grouped by gene features such as introns, exons, UTRs, etc.\n")
```

```{r getTargetedGenesTable, eval=annotationSummary} 
dt <- getTargetedGenesTable(queryRegions = queryRegions, 
                           txdbFeatures = txdbFeatures)
dt <- dt[order(transcripts, decreasing = TRUE)]

DT::datatable(dt[1:100], 
          extensions = c('Buttons', 'FixedColumns'), 
          options = list(fixedColumns = TRUE, 
                         scrollX = TRUE,
                         dom = 'Bfrtip',
                         buttons = c('copy', 'print', 'csv','excel', 'pdf')),
          filter = 'bottom'
          )
if(params$printProcessedTables == TRUE) {
  write.table(x = dt, file=paste0('Table',tableCount,'.getTargetedGenesTable.data.tsv'), quote = FALSE, sep = '\t', row.names = TRUE)
} 
tableCount <- tableCount + 1
```

```{r query_gene_types_figcap, results='asis', eval=annotationSummary}
cat("## Distribution of query regions in the genome grouped by gene types\n")
cat("**Figure",figureCount,":** The number of query regions that overlap different kinds of gene types are counted. The 'x' axis denotes the types of genes included in the analysis and the 'y' axis denotes the percentage of query regions (out of total number of query regions denoted with 'n') that overlap at least one genomic interval that host the corresponding gene type. If the query regions don't overlap any known genes, they are classified as 'Unknown'.\n")
```

```{r query_gene_types, eval=annotationSummary}
biotype_col <- grep('gene_biotype', colnames(overlaps.dt), value = T)[1]
if(!is.na(biotype_col)) {
  message("Using ",biotype_col," field in the GTF file for gene biotype plot")
  df <- overlaps.dt[,length(unique(queryIndex)), by = biotype_col]
  colnames(df) <- c("feature", "count")
  df$percent <- round(df$count / length(queryRegions) * 100, 1)
  df <- df[order(count, decreasing = TRUE)]
  p <- plotly::plot_ly(data = df, 
               type = "bar",
               x = df$feature,
               y = df$percent,
               text = paste("count:", df$count), color=df$feature)
  plotly::layout(p = p, 
         margin = list(l=100, r=100, b=150), 
         xaxis = list(showticklabels = TRUE,  tickangle = 90), 
         yaxis = list(title = paste("% query,", 
                                    "n =", length(queryRegions))),
         font = list(size = 14))
  if(params$printProcessedTables == TRUE) {
    write.table(x = df, file=paste0('Figure',figureCount,'.query_gene_types.data.tsv'), quote = FALSE, sep = '\t', row.names = TRUE)
  } 
  figureCount <- figureCount + 1
} else {
  warning("Skipping gene biotype plot because no such field exists in input GTF file.")
}
```

```{r transcriptBoundaryCoverage_figcap, results='asis', eval = annotationSummary}
cat("## Coverage profile of query regions at/around Transcription Start/End Sites\n")
cat("**Figure",figureCount,":** The depth of coverage of query regions at and around Transcription Start/End Sites\n")
```

```{r transcriptBoundaryCoverage, eval = annotationSummary}
cvgF <- getFeatureBoundaryCoverage(queryRegions = queryRegions, featureCoords = txdbFeatures$transcripts, flankSize = 1000, boundaryType = 'fiveprime', sampleN = 10000)
cvgT <- getFeatureBoundaryCoverage(queryRegions = queryRegions, featureCoords = txdbFeatures$transcripts, flankSize = 1000, boundaryType = 'threeprime', sampleN = 10000)

plotFeatureBoundaryCoverage(cvgF = cvgF, cvgT = cvgT, featureName = 'transcripts')

if(params$printProcessedTables == TRUE) {
  write.table(x = cvgF, file=paste0('Figure',figureCount,'.transcriptBoundaryCoverage.fiveprime.data.tsv'), quote = FALSE, sep = '\t', row.names = TRUE)
  write.table(x = cvgT, file=paste0('Figure',figureCount,'.transcriptBoundaryCoverage.threeprime.data.tsv'), quote = FALSE, sep = '\t', row.names = TRUE)
} 

figureCount <- figureCount + 1
```

```{r exonIntronCoverage_figcap, results='asis', eval = annotationSummary}
cat("## Coverage profile of query regions at Exon - Intron Boundaries\n")
cat("**Figure",figureCount,":** The depth of coverage of query regions at exon - intron junctions\n")
```

```{r exonIntronCoverage, eval = annotationSummary}
#split the exons into corresponding transcripts
exons <- txdbFeatures$exons
exonCounts <- lengths(rtracklayer::split(exons, names(exons)))

#To find the internal exons, for each transcript, remove the first and last exons from the GRanges object
#thus we can make sure that each exon in the end we are looking at is adjacent to an intron
internalExons <- exons[exonCounts[names(exons)] != exons$exon_rank,]
internalExons <- internalExons[internalExons$exon_rank > 1,]

cvgF <- getFeatureBoundaryCoverage(queryRegions = queryRegions, featureCoords = internalExons, boundaryType = 'fiveprime', flankSize = 1000, sampleN = 10000)
cvgT <- getFeatureBoundaryCoverage(queryRegions = queryRegions, featureCoords = internalExons, boundaryType = 'threeprime', flankSize = 1000, sampleN = 10000)

plotFeatureBoundaryCoverage(cvgF = cvgF, cvgT = cvgT, featureName = 'Internal Exons')

if(params$printProcessedTables == TRUE) {
  write.table(x = cvgF, file=paste0('Figure',figureCount,'.exonIntronBoundaryCoverage.fiveprime.data.tsv'), quote = FALSE, sep = '\t', row.names = TRUE)
  write.table(x = cvgT, file=paste0('Figure',figureCount,'.exonIntronBoundaryCoverage.threeprime.data.tsv'), quote = FALSE, sep = '\t', row.names = TRUE)
} 
figureCount <- figureCount + 1
```

```{r coverageprofilelist_figcap, results='asis', eval=annotationSummary}
cat("## Coverage profile of query regions across the length of different gene features\n")
cat("**Figure",figureCount,":** The query regions are overlaid with the genomic coordinates of features. Each entry corresponding to a feature is divided into 100 bins of equal length and for each bin the number of query regions that cover the corresponding bin is counted. Features shorter than 100bp are excluded. Thus, a coverage profile is obtained based on the distribution of the query regions. Mean coverage score for each bin is represented with ribbons where the thickness of the ribbon indicates the 95% confidence interval (mean +- standard error of the mean x 1.96). The strandedness of the features are taken into account. The coverage profile is plotted in the 5' to 3' direction.\n")
```

```{r coverageprofilelist, eval=annotationSummary}
cvgList <- calculateCoverageProfileList(queryRegions = queryRegions, 
                                       targetRegionsList = txdbFeatures, 
                                       sampleN = 10000)

p <- plotly::plot_ly(data = cvgList, type = 'scatter', mode = 'lines')
for (f in unique(cvgList$feature)){
  data <- cvgList[cvgList$feature == f,]
  p <- plotly::add_trace(p = p, data = data, x = ~bins, y = ~meanCoverage, 
                 legendgroup = f, showlegend = FALSE, opacity = 1, color = f)
  p <- plotly::add_ribbons(p = p, data = data, x = ~bins, 
                   ymin = data$meanCoverage - data$standardError*1.96,
                   ymax = data$meanCoverage + data$standardError*1.96, 
                   legendgroup = f, 
                   name = f, color = f
                   )
}
plotly::layout(p, font = list(size = 14))

if(params$printProcessedTables == TRUE) {
  write.table(x = cvgList, file=paste0('Figure',figureCount,'.coverageprofilelist.data.tsv'), quote = FALSE, sep = '\t', row.names = TRUE)
} 
figureCount <- figureCount + 1
```

```{r plot_motif_results_figcap, results='asis', eval=motifAnalysis}
cat('# Motif Discovery Results \n')
```

```{r motif_analysis, results='hide', eval=motifAnalysis}
message("Started motif discovery module...")
motifData <- discoverFeatureSpecificMotifs(queryRegions = queryRegions,
                                           txdbFeatures = txdbFeatures, 
                                           resizeN = 15, 
                                           sampleN = 10000,
                                           genomeVersion = params$genomeVersion,
                                           motifN = 1, 
                                           nCores = 1)
motifStats <- lapply(motifData, function(x) {
  df <- subset(getMotifSummaryTable(x), select = c('queryFraction', 'controlFraction'))
  df$pattern <- rownames(df)
  return(reshape2::melt(df, id.vars = 'pattern'))
})
```

```{r motif_analysis_table, results = 'asis', eval=motifAnalysis} 
cat("## Summary stats for top discovered motifs\n")
cat("**Table",tableCount,":** Top discriminative motif discovered for each transcript feature")  
# drop 'patterns' column, keep rownames with feature types
dt <- data.table(do.call(rbind, lapply(names(motifData), function(f) { 
  s <- getMotifSummaryTable(motifData[[f]])
  s$Feature <- f
  return(s)
})))

# reorder columns
dt <- dt[,c('Feature', 'patterns', 'queryFraction', 'controlFraction', 'oddsRatio', 'pvalue', 
            'querySeqs', 'controlSeqs', 'queryHits', 'controlHits')]
DT::datatable(dt, 
          extensions = c('Buttons', 'FixedColumns'), 
          options = list(fixedColumns = TRUE, 
                         scrollX = TRUE,
                         dom = 'Bfrtip',
                         buttons = c('copy', 'print', 'csv','excel', 'pdf')),
          filter = 'bottom'
          )
if(params$printProcessedTables == TRUE) {
  write.table(x = dt, file=paste0('Table',tableCount,'.motifSummaryTable.tsv'), quote = FALSE, sep = '\t', row.names = TRUE)
} 
tableCount <- tableCount + 1
```

```{r motif_analysis_plot_figcap, results='asis', eval=motifAnalysis}
cat('## Top motifs discovered in the sequences of the query regions {.tabset}\n')
for (i in 1:length(motifData)) {
  cat("### ",names(motifData)[i],"\n")
  print(seqLogo::seqLogo(getPWM(motifData[[i]]$matches_query[[1]])))
  cat('\n\n')
}
figureCount <- figureCount + 1
```


```{r GOanalysis, results='asis', eval=goAnalysis}
cat("# Functional Term Enrichment Analysis Results\n")
GP <- findEnrichedFunctions(targetedGenes, params$species)
#order by p-value
#GP <- GP[order(GP$p.value),]

cat("**Table",tableCount,":** Significant terms (FDR < 0.1) 
    enriched for genes that overlap query regions\n")  

#only display top GO terms in the HTML report.
max <- ifelse(nrow(GP) > 1000, 1000, nrow(GP))

DT::datatable(GP[1:max,], 
        extensions = c('Buttons', 'FixedColumns', 'Scroller'), 
        options = list(fixedColumns = TRUE, 
                       scrollY = 400,
                       scrollX = TRUE,
                       scroller = TRUE,
                       dom = 'Bfrtip',
                       buttons = c('colvis', 'copy', 'print', 'csv','excel', 'pdf'), 
                       columnDefs = list(
                         list(targets = c(0,1,2,5,9,11,13,14), visible = FALSE)
                       )
                       ),
        filter = 'bottom'
        )

tableCount <- tableCount + 1
```

# Acknowledgements

RCAS is developed in the group of 
[Altuna Akalin](http://bioinformatics.mdc-berlin.de/team.html#altuna-akalin-phd)
(head of the Scientific Bioinformatics Platform) by
[Bora Uyar](http://bioinformatics.mdc-berlin.de/team.html#bora-uyar-phd)
(Bioinformatics Scientist),
[Dilmurat Yusuf](http://bioinformatics.mdc-berlin.de/team.html#dilmurat-yusuf-phd)
(Bioinformatics Scientist) and 
[Ricardo Wurmus](http://bioinformatics.mdc-berlin.de/team.html#ricardo-wurmus)
(System Administrator) at the Berlin Institute of Medical Systems Biology
([BIMSB](https://www.mdc-berlin.de/13800178/en/bimsb))
at the Max-Delbrueck-Center for Molecular Medicine
([MDC](https://www.mdc-berlin.de)) in Berlin.

RCAS is developed as a bioinformatics service as part of
the [RNA Bioinformatics Center](http://www.denbi.de/index.php/rbc),
which is one of the eight centers of
the German Network for Bioinformatics Infrastructure
([de.NBI](http://www.denbi.de/)).  

# Session Information
```{r sessionInfo}
sessionInfo()
```

---
title: "The RNA Centric Analysis System Report"
author: "Bora Uyar, Dilmurat Yusuf, Ricardo Wurmus, Altuna Akalin"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction - single sample analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r global_options, include=FALSE}
knitr::opts_chunk$set(warning = FALSE, message = FALSE, eval = TRUE, fig.width = 7, fig.height = 4.2)
```


```{r load_libraries, results='hide'}
library(RCAS)
```

**Warning**: In this vignette, due to space limitations, we demonstrate the functions of RCAS using static images. In order to see how an interactive report from RCAS looks see `RCAS::runReport()`. 

For the most up-to-date functionality, usage and installation instructions, and example outputs, see our [github repository here](https://github.com/BIMSBbioinfo/RCAS). 

# Introduction

RCAS is an automated system that provides dynamic genome annotations for custom
input files that contain transcriptomic regions. Such transcriptomic regions
could be, for instance, peak regions detected by CLIP-Seq analysis that detect
protein-RNA interactions, RNA modifications (alias the epitranscriptome),
CAGE-tag locations, or any other collection of target regions at the level of
the transcriptome.

RCAS is designed as a reporting tool for the functional analysis of RNA-binding
sites detected by high-throughput experiments. It takes as input a BED format
file containing the genomic coordinates of the RNA binding sites and a GTF file
that contains the genomic annotation features usually provided by publicly
available databases such as Ensembl and UCSC. RCAS performs overlap operations
between the genomic coordinates of the RNA binding sites and the genomic
annotation features and produces in-depth annotation summaries such as the
distribution of binding sites with respect to gene features (exons, introns,
5’/3’ UTR regions, exon-intron boundaries, promoter regions, and whole
transcripts) along with functionally enriched term for targeted genes. Moreover,
RCAS can darry out discriminative motif discovery in the query regions. The
final report of RCAS consists of high-quality dynamic figures and tables, which
are readily applicable for publications or other academic usage.

#  Data input

RCAS minimally requires as input a BED file and a GTF file.
The BED file should contain coordinates/intervals
of transcriptomic regions
which are located via transcriptomics methods such as Clip-Seq.
The GTF file should provide reference annotation.
The recommended source of GTF files is the 
[ENSEMBLE](http://www.ensembl.org/info/data/ftp/index.html) database. 

For this vignette, in order to demonstrate RCAS functionality,
we use sample BED and GTF data that are built-in the RCAS library,
which can be imported using a common R function: data().
To import custom BED and GTF files,
the user should execute two RCAS functions called importBed() and importGtf().

## Importing sample data
```{r sample_data}
library(RCAS)
data(queryRegions) #sample queryRegions in BED format()
data(gff)          #sample GFF file
```

## Importing custom data
To use importBed() and importGtf(),
the user should provide file paths to the respective
BED file and GTF file.
To reduce memory usage and time consumption,
we advise the user to set `sampleN=10000` to avoid
huge input of intervals.

```{r RCAS_import_data, eval = FALSE}
queryRegions <- importBed(filePath = <path to BED file>, sampleN = 10000)
gff <- importGtf(filePath = <path to GTF file>)
```

# Summarizing Overlaps of Query Regions with Genomic Annotation Features

## Querying the annotation file 
```{r queryGFF}
overlaps <- as.data.table(queryGff(queryRegions = queryRegions, gffData = gff))
```


### Finding targeted gene types
To find out the distribution of the query regions across gene types:
```{r query_gene_types}
biotype_col <- grep('gene_biotype', colnames(overlaps), value = T)
df <- overlaps[,length(unique(queryIndex)), by = biotype_col]
colnames(df) <- c("feature", "count")
df$percent <- round(df$count / length(queryRegions) * 100, 1)
df <- df[order(count, decreasing = TRUE)]
ggplot2::ggplot(df, aes(x = reorder(feature, -percent), y = percent)) + 
  geom_bar(stat = 'identity', aes(fill = feature)) + 
  geom_label(aes(y = percent + 0.5), label = df$count) + 
  labs(x = 'transcript feature', y = paste0('percent overlap (n = ', length(queryRegions), ')')) + 
  theme_bw(base_size = 14) + 
  theme(axis.text.x = element_text(angle = 90))
```

## Extending the annotation feature space 

GTF files contain some annotation features (e.g. exons, transcripts) that are 
usually explicitly defined, however, some transcript features such as introns,
exon-intron boundaries, promoter regions are only implicitly defined. Such 
implicit features can be extracted from a GTF file using makeTxDb family of 
functions from the txdbmaker library.

First we create a list of GRanges objects, where each list element contains all
the available coordinates of transcript features such as transcripts, exons, introns,
5'/3' UTRs, exon-intron boundaries, and promoter regions. 

```{r getTxdbFeatures}
txdbFeatures <- getTxdbFeaturesFromGRanges(gff)
```

### Plotting overlap counts between query regions and transcript features
To have a global overview of the distribution of query regions across gene
features, we can use the summarizeQueryRegions function. If a given query region 
does not overlap with any of the given coordinates of the transcript features, 
it is categorized under `NoFeatures`. 

```{R summarizeQueryRegions}
summary <- summarizeQueryRegions(queryRegions = queryRegions, 
                                 txdbFeatures = txdbFeatures)

df <- data.frame(summary)
df$percent <- round((df$count / length(queryRegions)), 3) * 100
df$feature <- rownames(df)
ggplot2::ggplot(df, aes(x = reorder(feature, -percent), y = percent)) + 
  geom_bar(stat = 'identity', aes(fill = feature)) + 
  geom_label(aes(y = percent + 3), label = df$count) + 
  labs(x = 'transcript feature', y = paste0('percent overlap (n = ', length(queryRegions), ')')) + 
  theme_bw(base_size = 14) + 
  theme(axis.text.x = element_text(angle = 90))
```

### Obtaining a table of overlap counts between query regions and genes 
To find out which genes overlap with how many queries and categorise overlaps by
transcript features; we use `getTargetedGenesTable` function, which returns a data.frame
object.  
```{r getTargetedGenesTable}
dt <- getTargetedGenesTable(queryRegions = queryRegions, 
                           txdbFeatures = txdbFeatures)
dt <- dt[order(transcripts, decreasing = TRUE)]

knitr::kable(dt[1:10,])
```

### Profiling the coverage of query regions across transcript features 

#### Coverage profile of query regions at feature boundaries 

It may be useful to look at the distribution of query regions at the boundaries of transcript features.
For instance, it may be important to see the relative signal at transcript ends (transcription start sites
versus transcription end sites). Or, it may be important to see how the signal is distributed at exon boundaries, which may give an idea about the regulation of the transcript. Here we demonstrate how to
get such signal distributions at transcription start/end sites. The same approach can be done for any other
collection of transcript features (exons, introns, promoters, UTRs etc.)

```{r transcriptBoundaryCoverage}
cvgF <- getFeatureBoundaryCoverage(queryRegions = queryRegions, 
                                   featureCoords = txdbFeatures$transcripts, 
                                   flankSize = 1000, 
                                   boundaryType = 'fiveprime', 
                                   sampleN = 10000)
cvgT <- getFeatureBoundaryCoverage(queryRegions = queryRegions, 
                                   featureCoords = txdbFeatures$transcripts, 
                                   flankSize = 1000, 
                                   boundaryType = 'threeprime', 
                                   sampleN = 10000)

cvgF$boundary <- 'fiveprime'
cvgT$boundary <- 'threeprime'

df <- rbind(cvgF, cvgT)

ggplot2::ggplot(df, aes(x = bases, y = meanCoverage)) + 
  geom_ribbon(fill = 'lightgreen', 
              aes(ymin = meanCoverage - standardError * 1.96, 
                  ymax = meanCoverage + standardError * 1.96)) + 
 geom_line(color = 'black') + 
 facet_grid( ~ boundary) + theme_bw(base_size = 14) 

```

#### Coverage profile of query regions for all transcript features
Coverage profiles can be obtained for a single type of transcript feature or a list of
transcript features. Here we demonstrate how to get coverage profile of query regions 
across all available transcript features. It might be a good idea to use sampleN
parameter to randomly downsample the target regions to speed up the 
calculations.

```{r coverageprofilelist, fig.height=6}
cvgList <- calculateCoverageProfileList(queryRegions = queryRegions, 
                                       targetRegionsList = txdbFeatures, 
                                       sampleN = 10000)

ggplot2::ggplot(cvgList, aes(x = bins, y = meanCoverage)) + 
  geom_ribbon(fill = 'lightgreen', 
              aes(ymin = meanCoverage - standardError * 1.96, 
                  ymax = meanCoverage + standardError * 1.96)) + 
 geom_line(color = 'black') + theme_bw(base_size = 14) +
 facet_wrap( ~ feature, ncol = 3) 
```

# Discriminative Motif Discovery

## Calculating enriched motifs

We build a classifier based on k-mer frequencies (allowing for mismatches) to 
find the most informative motifs that help discriminate the query sequences from the 
background distribution. 

```{r motif_analysis}
motifResults <- runMotifDiscovery(queryRegions = queryRegions, 
                           resizeN = 15, sampleN = 10000,
                           genomeVersion = 'hg19', motifWidth = 6,
                           motifN = 1, nCores = 1)

seqLogo::seqLogo(getPWM(motifResults$matches_query[[1]]))
```

## motif analysis: getting motif summary statistics
A summary table from the motif analysis results can be obtained
```{r motif_analysis_table}
summary <- getMotifSummaryTable(motifResults)
knitr::kable(summary)
```


# Functional enrichment  analysis

RCAS makes use of [gprofiler2](https://cran.r-project.org/web/packages/gprofiler2/index.html) package 
enriched functions in genes that overlap the query regions. 

```{r GO analysis}
targetedGenes <- unique(overlaps$gene_id)

res <- RCAS::findEnrichedFunctions(targetGenes = targetedGenes, species = 'hsapiens')
res <- res[order(res$p_value),]
resGO <- res[grep('GO:BP', res$source),]
knitr::kable(subset(resGO[1:10,], select = c('p_value', 'term_name', 'source')))
```

# Generating a full report 

The users can use the runReport() function to generate full custom reports 
including all the analysis modules described above. There are four main parts of
the analysis report.

- Annotation summaries via overlap operations 
- Functional term analysis 
- Motif analysis

By default, runReport() function aims to run all three modules, while the user
can turn off these individual modules.

Below are example commands to generate reports using these functionalities.

## A test run for human

```
runReport()
```
## A custom run for human
```
runReport( queryFilePath = 'input.BED',
            gffFilePath = 'annotation.gtf')
```
## To turn off certain modules of the report
``` 
runReport( queryFilePath = 'input.BED',
            gffFilePath = 'annotation.gtf',
            motifAnalysis = FALSE,
            goAnalysis = FALSE )
```
## To run the pipeline for species other than human

You can run RCAS for any of the genome versions available in the BSgenome package.  
See `BSgenome::available.genomes`. 

```
runReport( queryFilePath = 'input.mm9.BED',
            gffFilePath = 'annotation.mm9.gtf',
            genomeVersion = 'mm9' )
```

## To turn off verbose output and progress bars
```
runReport(quiet = TRUE)
```

## Printing raw data generated by the runReport function
One may be interested in printing the raw data used to make the plots and tables
in the HTML report output of runReport function. Such tables could be used for
meta-analysis of multiple analysis results. In order to activate this function,
printProcessedTables argument must be set to TRUE.
```
runReport(printProcessedTables = TRUE)
```

# Acknowledgements

RCAS is developed in the group of 
[Altuna Akalin](http://bioinformatics.mdc-berlin.de/team.html#altuna-akalin-phd)
(head of the Scientific Bioinformatics Platform) by
[Bora Uyar](http://bioinformatics.mdc-berlin.de/team.html#bora-uyar-phd)
(Bioinformatics Scientist),
[Dilmurat Yusuf](http://bioinformatics.mdc-berlin.de/team.html#dilmurat-yusuf-phd)
(Bioinformatics Scientist) and 
[Ricardo Wurmus](http://bioinformatics.mdc-berlin.de/team.html#ricardo-wurmus)
(System Administrator) at the Berlin Institute of Medical Systems Biology
([BIMSB](https://www.mdc-berlin.de/13800178/en/bimsb))
at the Max-Delbrueck-Center for Molecular Medicine
([MDC](https://www.mdc-berlin.de)) in Berlin.

RCAS is developed as a bioinformatics service as part of
the [RNA Bioinformatics Center](http://www.denbi.de/index.php/rbc),
which is one of the eight centers of
the German Network for Bioinformatics Infrastructure
([de.NBI](http://www.denbi.de/)).  

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/databaseSetUpFunctions.R
\name{createDB}
\alias{createDB}
\title{createDB}
\usage{
createDB(
  dbPath = file.path(getwd(), "rcasDB.sqlite"),
  projDataFile,
  gtfFilePath = "",
  update = FALSE,
  genomeVersion,
  annotationSummary = TRUE,
  coverageProfiles = TRUE,
  motifAnalysis = TRUE,
  nodeN = 1
)
}
\arguments{
\item{dbPath}{Path to the sqlite database file (could be an existing file or 
a new file path to be created at the given path)}

\item{projDataFile}{A file consisting of meta-data about the input samples. 
Must minimally consist of two columns: 1. sampleName (name of the sample) 
2. bedFilePath (full path to the location of the BED file containing data 
for the sample)}

\item{gtfFilePath}{Path to the GTF file (preferably downloaded from the 
Ensembl database) that contains genome annotations}

\item{update}{TRUE/FALSE (default: FALSE) whether an existing database 
should be updated}

\item{genomeVersion}{A character string to denote for which genome version 
the analysis is being done. Available options are hg19/hg38 (human), mm9/mm10 
(mouse), ce10 (worm) and dm3 (fly).}

\item{annotationSummary}{TRUE/FALSE (default:TRUE) whether annotation summary
module should be run}

\item{coverageProfiles}{TRUE/FALSE (default: TRUE) whether coverage profiles
module should be run}

\item{motifAnalysis}{TRUE/FALSE (default: TRUE) whether motif discovery module
should be run}

\item{nodeN}{Number of cpus to use for parallel processing (default: 1)}
}
\value{
Path to an SQLiteConnection object created by RSQLite package
}
\description{
Creates an sqlite database consisting of various tables of data obtained 
from processed BED files
}
\examples{
FUS_path <- system.file("extdata", "FUS_Nakaya2013c_hg19.bed",
package='RCAS') 

FMR1_path <- system.file("extdata",
"FMR1_Ascano2012a_hg19.bed", package='RCAS') 

projData <- data.frame('sampleName' = c('FUS', 'FMR1'), 
'bedFilePath' = c(FUS_path,FMR1_path), stringsAsFactors = FALSE)
 
write.table(projData, 'myProjDataFile.tsv', sep = '\t', quote =FALSE, 
row.names = FALSE) 

gtfFilePath <- system.file("extdata",
"hg19.sample.gtf", package='RCAS')

createDB(dbPath = 'hg19.RCASDB.sqlite', 
projDataFile = './myProjDataFile.tsv', 
gtfFilePath = gtfFilePath,  
genomeVersion = 'hg19', 
motifAnalysis = FALSE, 
coverageProfiles = FALSE)

#Note: to add new data to an existing database, set update = TRUE

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_functions.R
\name{calculateCoverageProfileFromTxdb}
\alias{calculateCoverageProfileFromTxdb}
\title{calculateCoverageProfileFromTxdb}
\usage{
calculateCoverageProfileFromTxdb()
}
\description{
This function is deprecated. Use ?calculateCoverageProfile instead.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta_analysis_functions.R
\name{getIntervalOverlapMatrix}
\alias{getIntervalOverlapMatrix}
\title{getIntervalOverlapMatrix}
\usage{
getIntervalOverlapMatrix(
  queryRegionsList,
  targetRegions,
  targetRegionNames = NULL,
  nodeN = 1
)
}
\arguments{
\item{queryRegionsList}{A GRangesList object}

\item{targetRegions}{A GRanges object}

\item{targetRegionNames}{Optional vector of names to be used as rownames in
the resulting matrix. The vector indices must correspond to the intervals
in targetRegions object.}

\item{nodeN}{Positive integer value to use one or more cpus for parallel
computation (default: 1)}
}
\value{
A binary matrix object consisting of number of rows equal to the
  number of intervals in targetRegions object, and number of columns equal to
  the number of GRanges objects available in the queryRegionsList object.
}
\description{
This function is used to obtain a binary matrix of overlaps between a list of
GRanges objects (GRangesList object) and a target GRanges object. The
resulting matrix has N rows where N is the number of intervals in the target
GRanges object and M columns where M is the number GRanges objects in the
query GRangesList object.
}
\examples{
data(gff)
input1 <- system.file("extdata", "testfile.bed", package='RCAS') 
input2 <- system.file("extdata", "testfile2.bed", package='RCAS') 
bedData <- RCAS::importBedFiles(filePaths = c(input1, input2)) 
M <- RCAS::getIntervalOverlapMatrix(
queryRegionsList = bedData, 
targetRegions = gff[gff$type == 'gene',][1:100], 
targetRegionNames = gff[gff$type == 'gene',][1:100]$gene_name)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_functions.R
\name{runReport}
\alias{runReport}
\title{Generate a RCAS Report for a list of transcriptome-level segments}
\usage{
runReport(
  queryFilePath = "testdata",
  gffFilePath = "testdata",
  annotationSummary = TRUE,
  goAnalysis = TRUE,
  motifAnalysis = TRUE,
  genomeVersion = "hg19",
  outDir = getwd(),
  printProcessedTables = FALSE,
  sampleN = 0,
  quiet = FALSE,
  selfContained = TRUE
)
}
\arguments{
\item{queryFilePath}{a BED format file which contains genomic coordinates of 
protein-RNA binding sites}

\item{gffFilePath}{A GTF format file which contains genome annotations 
(preferably from ENSEMBL)}

\item{annotationSummary}{TRUE/FALSE (default: TRUE) A switch to decide if 
RCAS should provide annotation summaries from overlap operations}

\item{goAnalysis}{TRUE/FALSE (default: TRUE) A switch to decide if RCAS 
should run GO term enrichment analysis}

\item{motifAnalysis}{TRUE/FALSE (default: TRUE) A switch to decide if RCAS 
should run motif analysis}

\item{genomeVersion}{A character string to denote for which genome version 
the analysis is being done.}

\item{outDir}{Path to the output directory. (default: current working 
directory)}

\item{printProcessedTables}{boolean value (default: FALSE). If set to TRUE, 
raw data tables that are used for plots/tables will be printed to text
files.}

\item{sampleN}{integer value (default: 0). A parameter to determine if the 
input query regions should be downsampled to a smaller size in order to 
make report generation quicker. When set to 0, downsampling won't be done. 
To activate the sampling a positive integer value that is smaller than the
total number of query regions should be given.}

\item{quiet}{boolean value (default: FALSE). If set to TRUE, progress bars 
and chunk labels will be suppressed while knitting the Rmd file.}

\item{selfContained}{boolean value (default: TRUE). By default, the generated
html file will be self-contained, which means that all figures and tables 
will be embedded in a single html file with no external dependencies 
(See rmarkdown::html_document)}
}
\value{
An html generated using rmarkdown/knitr/pandoc that contains 
  interactive figures, tables, and text that provide an overview of the 
  experiment
}
\description{
This is the main report generation function for RCAS. This function takes a
BED file, a GTF file to create a summary report regarding the annotation data
that overlap the input BED file, enrichment analysis for functional terms,
and motif analysis.
}
\examples{
#Default run will generate a report using built-in test data for hg19 genome.
\dontrun{
runReport()
}

#A custom run for human
\dontrun{
runReport( queryFilePath = 'input.BED',
           gffFilePath = 'annotation.gtf', 
           genomeVersion = 'hg19')
           }
# To turn off certain modules of the report
\dontrun{
runReport( queryFilePath = 'input.BED',
           gffFilePath = 'annotation.gtf',
           motifAnalysis = FALSE,
           goAnalysis = FALSE )
           }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_functions.R
\name{calculateCoverageProfile}
\alias{calculateCoverageProfile}
\title{calculateCoverageProfile}
\usage{
calculateCoverageProfile(
  queryRegions,
  targetRegions,
  sampleN = 0,
  bin.num = 100,
  bin.op = "mean",
  strand.aware = TRUE
)
}
\arguments{
\item{queryRegions}{GRanges object imported from a BED file using 
\code{importBed} function}

\item{targetRegions}{GRanges object containing genomic coordinates of a 
target feature (e.g. exons)}

\item{sampleN}{If set to a positive integer, \code{targetRegions} will be 
downsampled to \code{sampleN} regions}

\item{bin.num}{Positive integer value (default: 100) to determine how many
bins the targetRegions should be split into (See
genomation::ScoreMatrixBin)}

\item{bin.op}{The operation to apply for each bin: 'min', 'max', or 'mean'
(default: mean). (See genomation::ScoreMatrixBin)}

\item{strand.aware}{TRUE/FALSE (default: TRUE) The strands of target regions
are considered.}
}
\value{
A ScoreMatrix object returned by \code{genomation::ScoreMatrixBin} 
  function. Target regions are divided into 100 equal sized bins and coverage
  level is calculated in a strand-specific manner.
}
\description{
This function checks overlaps between input query regions and annotation 
features, and then calculates coverage profile along target regions.
}
\examples{
data(gff)
data(queryRegions)
txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)
df <- calculateCoverageProfile(queryRegions = queryRegions,
                              targetRegions = txdbFeatures$exons,
                                    sampleN = 1000)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_analysis.R
\name{discoverFeatureSpecificMotifs}
\alias{discoverFeatureSpecificMotifs}
\title{discoverFeatureSpecificMotifs}
\usage{
discoverFeatureSpecificMotifs(queryRegions, txdbFeatures, ...)
}
\arguments{
\item{queryRegions}{GRanges object containing coordinates of input query
regions imported by the \code{\link{importBed}} function}

\item{txdbFeatures}{A list of GRanges objects where each GRanges object
corresponds to the genomic coordinates of gene features such as promoters,
introns, exons, 5'/3' UTRs and whole transcripts.
This list of GRanges objects are obtained by the function
\code{\link{getTxdbFeaturesFromGRanges}} or \code{\link{getTxdbFeatures}}.}

\item{...}{Other arguments passed to \code{\link{runMotifRG}} function.
Important arguments are 'genomeVersion' and motifN. If motifN is bigger
than 1, then multiple motifs will be found but only the top motif will be
plotted.}
}
\value{
A data.frame object
}
\description{
This function groups query regions based on their overlap with different
transcript features and generates a table of top enriched motif and matching
patterns for each given transcript feature type along with some other motif 
discovery related statistics.
}
\examples{
\dontrun{
data(gff)
data(queryRegions)
txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)
discoverFeatureSpecificMotifs(queryRegions = queryRegions, 
genomeVersion = 'hg19', txdbFeatures = txdbFeatures, 
motifN = 1, nCores = 1)}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_functions.R
\name{calculateCoverageProfileListFromTxdb}
\alias{calculateCoverageProfileListFromTxdb}
\title{calculateCoverageProfileListFromTxdb}
\usage{
calculateCoverageProfileListFromTxdb()
}
\description{
This function is deprecated. Use ?calculateCoverageProfileList instead.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_functions.R
\name{summarizeQueryRegions}
\alias{summarizeQueryRegions}
\title{summarizeQueryRegions}
\usage{
summarizeQueryRegions(queryRegions, txdbFeatures)
}
\arguments{
\item{queryRegions}{GRanges object imported from a BED file using
\code{importBed} function}

\item{txdbFeatures}{List of GRanges objects - outputs of
\code{getTxdbFeaturesFromGRanges} and \code{getTxdbFeatures} functions}
}
\value{
A data frame with two columns where first column holds features and
  second column holds corresponding counts
}
\description{
This function counts number of query regions that overlap with different
types of gene features.
}
\examples{
data(gff)
data(queryRegions)
txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)
summary <- summarizeQueryRegions(queryRegions = queryRegions,
                                 txdbFeatures = txdbFeatures)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting_functions.R
\name{plotFeatureBoundaryCoverage}
\alias{plotFeatureBoundaryCoverage}
\title{plotFeatureBoundaryCoverage}
\usage{
plotFeatureBoundaryCoverage(cvgF, cvgT, featureName)
}
\arguments{
\item{cvgF}{data.frame object containing 'fiveprime' coverage data returned 
by getFeatureBoundaryCoverage function}

\item{cvgT}{data.fram object containing 'threeprime' coverage data returned 
by getFeatureBoundaryCoverage function}

\item{featureName}{character object. This is used to label the axes (e.g.
transcripts, exons)}
}
\value{
a plotly htmlwidget is returned
}
\description{
This function is used to create interactive plots displaying 5' and 3'
end coverage profiles of given transcript features.
}
\examples{
data(queryRegions)
data(gff)
txdb <- txdbmaker::makeTxDbFromGRanges(gff)
transcriptCoords <- GenomicFeatures::transcripts(txdb)
cvgF <- getFeatureBoundaryCoverage (queryRegions = queryRegions,
                                    featureCoords = transcriptCoords,
                                    flankSize = 100,
                                    boundaryType = 'fiveprime',
                                    sampleN = 1000)
                                    
cvgT <- getFeatureBoundaryCoverage (queryRegions = queryRegions,
                                    featureCoords = transcriptCoords,
                                    flankSize = 100,
                                    boundaryType = 'threeprime',
                                    sampleN = 1000)
p <- plotFeatureBoundaryCoverage(cvgF = cvgF, 
                                 cvgT = cvgT, 
                          featureName = 'transcript')   
                                                          
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_analysis.R
\name{extractSequences}
\alias{extractSequences}
\title{extractSequences}
\usage{
extractSequences(queryRegions, genomeVersion)
}
\arguments{
\item{queryRegions}{GRanges object containing coordinates of input query
regions imported by the \code{\link{importBed}} function}

\item{genomeVersion}{A character string to denote the BS genome library
required to extract sequences. Available options are hg19, mm9, ce10 and
dm3.}
}
\value{
DNAStringSet object will be returned
}
\description{
Given a GRanges object and a genome version (hg19, mm9, ce10 or dm3), this
function extracts the DNA sequences for all genomic regions found in an input
object.
}
\examples{

data(queryRegions)
sequences <- extractSequences(queryRegions = queryRegions,
                             genomeVersion = 'hg19')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment_analysis.R
\name{runTopGO}
\alias{runTopGO}
\title{runTopGO}
\usage{
runTopGO()
}
\description{
This function is deprecated. Use findEnrichedFunctions instead.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_functions.R
\name{getTxdbFeaturesFromGRanges}
\alias{getTxdbFeaturesFromGRanges}
\title{getTxdbFeaturesFromGRanges}
\usage{
getTxdbFeaturesFromGRanges(gffData)
}
\arguments{
\item{gffData}{A GRanges object imported by \code{importGtf} function}
}
\value{
A list of GRanges objects
}
\description{
This function takes as input a GRanges object that contains GTF file contents
(e.g from the output of \code{importGtf} function). 
Then extracts the coordinates of gene features
such as promoters, introns, exons, 5'/3' UTRs and
whole transcripts.
}
\examples{
data(gff)
txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_analysis.R
\name{createControlRegions}
\alias{createControlRegions}
\title{createControlRegions}
\usage{
createControlRegions(queryRegions)
}
\arguments{
\item{queryRegions}{GRanges object containing coordinates of input query
regions imported by the \code{\link{importBed}} function.}
}
\value{
GRanges object that contains the same number of regions as query
  regions
}
\description{
Given a GRanges object of query regions, create a background set of peaks
that have the same length distribution based on the flanking regions of the
peaks.
}
\examples{
data(queryRegions)
controlRegions <- createControlRegions(queryRegions = queryRegions)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_functions.R
\name{checkSeqDb}
\alias{checkSeqDb}
\title{checkSeqDb}
\usage{
checkSeqDb(genomeVersion)
}
\arguments{
\item{genomeVersion}{String that denotes genome version. 
To unambigously select a BSgenome object, provide a string
that matches the end of the available genomes at: 
BSgenome::available.genomes().}
}
\value{
Returns a BSgenome object that uniquely matches the 
genomeVersion.
}
\description{
Given a string that denotes a genome version (e.g. hg19)
returns the BSgenome object matching the genome version 
that are available in BSgenome::available.genomes()
}
\examples{
checkSeqDb('hg19')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_analysis.R
\name{generateKmers}
\alias{generateKmers}
\title{Generate K-mers}
\usage{
generateKmers(k, letters = c("A", "C", "G", "T"))
}
\arguments{
\item{k}{The length of the strings to be generated}

\item{letters}{A character vector}
}
\value{
Vector of strings
}
\description{
Given a list of characters, generates all possible fixed length strings
}
\examples{
generateKmers(3, c('A', 'C', 'G'))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_analysis.R
\name{findDifferentialMotifs}
\alias{findDifferentialMotifs}
\title{Find Differential Motifs}
\usage{
findDifferentialMotifs(
  querySeqs,
  controlSeqs,
  motifWidth = 6,
  motifN = 1,
  nCores = 1,
  maxMismatch = 1
)
}
\arguments{
\item{querySeqs}{A DNAStringSet object that is the regions of interest.}

\item{controlSeqs}{A DNAStrintSet object that serve as the control}

\item{motifWidth}{A Positive integer (default: 6) for the generated k-mers. Warning: we recommend
using values below 10 as the computation gets exponentially difficult as the 
motif width is increased.}

\item{motifN}{A positive integer (default:1) denoting the maximum number of 
motifs that should be returned by the \code{findDifferentialMotifs} function}

\item{nCores}{A positive integer (default:1) number of cores used for 
parallel execution.}

\item{maxMismatch}{A positive integer (default: 1) - maximum number of
mismatches to allow when searching for k-mer matches in sequences.}
}
\description{
Find Differential Motifs
}
\examples{
data(queryRegions)

# get query and control sequences
querySeqs <- extractSequences(queryRegions[1:500], 'hg19')
controlRegions <- createControlRegions(queryRegions[1:500])
controlSeqs <- extractSequences(controlRegions, 'hg19')

#run motif discovery
motifResults <- findDifferentialMotifs(querySeqs = querySeqs, 
                                       controlSeqs = controlSeqs, 
                                       motifWidth = 5,
                                       motifN = 1,
                                       maxMismatch = 0, 
                                       nCores = 1)
#summarize motif results
getMotifSummaryTable(motifResults)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta_analysis_functions.R
\name{importBedFiles}
\alias{importBedFiles}
\title{importBedFiles}
\usage{
importBedFiles(filePaths, ...)
}
\arguments{
\item{filePaths}{A vector of paths to one or more BED files}

\item{...}{Other parameters passed to RCAS::importBed and
rtracklayer::import.bed function}
}
\value{
A \code{GRangesList} object containing the coordinates of the 
  intervals from multiple input BED files
}
\description{
This function is a wrapper that uses \code{RCAS::importBed()} function to 
import BED files as a GRangesList object
}
\examples{
input1 <- system.file("extdata", "testfile.bed", package='RCAS') 
input2 <- system.file("extdata", "testfile2.bed", package='RCAS') 
bedData <- importBedFiles(filePaths = c(input1, input2), 
keepStandardChr = TRUE) 
# when importing multiple bed files with different column names, it 
# is required to pass the common column names to be parsed from the 
# bed files
bedData <- importBedFiles(filePaths = c(input1, input2),
                 colnames = c('chrom', 'start', 'end', 'strand'))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta_analysis_functions.R
\name{getFeatureBoundaryCoverageMulti}
\alias{getFeatureBoundaryCoverageMulti}
\title{getFeatureBoundaryCoverageMulti}
\usage{
getFeatureBoundaryCoverageMulti(bedData, txdbFeatures, sampleN = 10000)
}
\arguments{
\item{bedData}{GRangesList object imported from multiple BED files 
using \code{importBedFiles} function}

\item{txdbFeatures}{List of GRanges objects - outputs of 
\code{getTxdbFeaturesFromGRanges} and \code{getTxdbFeatures} functions}

\item{sampleN}{(default=10000) Positive integer value that is used to 
randomly down-sample the target feature coordinates to improve the runtime.
Set to 0 to avoid downsampling.}
}
\value{
A data.frame object with coverage data at three prime and five prime
  boundaries of a list of transcript features
}
\description{
This function is a wrapper function to run RCAS::getFeatureBoundaryCoverage 
multiple times, which is useful to get coverage signals across different 
kinds of transcript features for a given list of bed files imported as a 
GRangesList object.
}
\examples{
data(gff)
data(queryRegions)
queryRegionsList <- GenomicRanges::GRangesList(queryRegions, queryRegions)
names(queryRegionsList) <- c('q1', 'q2')
txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)
getFeatureBoundaryCoverageMulti(queryRegionsList, txdbFeatures, sampleN = 500)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_functions.R
\name{getTargetedGenesTable}
\alias{getTargetedGenesTable}
\title{getTargetedGenesTable}
\usage{
getTargetedGenesTable(queryRegions, txdbFeatures)
}
\arguments{
\item{queryRegions}{GRanges object containing coordinates of input query
regions imported by the \code{\link{importBed}} function}

\item{txdbFeatures}{A list of GRanges objects where each GRanges object
corresponds to the genomic coordinates of gene features such as promoters,
introns, exons, 5'/3' UTRs and whole transcripts.
This list of GRanges objects are obtained by the function
\code{\link{getTxdbFeaturesFromGRanges}} or \code{\link{getTxdbFeatures}}.}
}
\value{
A data.frame object where rows correspond to genes and columns
  correspond to gene features
}
\description{
This function provides a list of genes which are targeted by query regions
and their corresponding numbers from an input BED file. Then, the hits are
categorized by the gene features such as promoters, introns, exons,
5'/3' UTRs and whole transcripts.
}
\examples{
data(gff)
data(queryRegions)
txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)
featuresTable <- getTargetedGenesTable(queryRegions = queryRegions,
                                       txdbFeatures = txdbFeatures)

#or
\dontrun{
txdb <- txdbmaker::makeTxDbFromGRanges(gff)
txdbFeatures <- getTxdbFeatures(txdb)
featuresTable <- getTargetedGenesTable(queryRegions = queryRegions,
                                       txdbFeatures = txdbFeatures)
                                       }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment_analysis.R
\name{createOrthologousGeneSetList}
\alias{createOrthologousGeneSetList}
\title{createOrthologousMsigdbDataset
This function is deprecated. For functional enrichment 
analysis, use findEnrichedFunctions.}
\usage{
createOrthologousGeneSetList()
}
\description{
createOrthologousMsigdbDataset
This function is deprecated. For functional enrichment 
analysis, use findEnrichedFunctions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_functions.R
\name{importBed}
\alias{importBed}
\title{importBed}
\usage{
importBed(filePath, sampleN = 0, keepStandardChr = TRUE, debug = TRUE, ...)
}
\arguments{
\item{filePath}{Path to a BED file}

\item{sampleN}{A positive integer value. The number of intervals in the input
BED file are randomly downsampled to include intervals as many as
\code{sampleN}. The input will be downsampled only if this value is larger
than zero and less than the total number of input intervals.}

\item{keepStandardChr}{TRUE/FALSE (default:TRUE). If set to TRUE, will
convert the \code{seqlevelsStyle} to 'UCSC' and apply
\code{keepStandardChromosomes} function to only keep data from the standard
chromosomes}

\item{debug}{TRUE/FALSE (default:TRUE). Set to FALSE to turn off messages}

\item{...}{Other arguments passed to rtracklayer::import.bed function}
}
\value{
A \code{GRanges} object containing the coordinates of the intervals
  from an input BED file
}
\description{
This function uses \code{rtracklayer::import.bed()} function to import BED
files
}
\examples{
input <- system.file("extdata", "testfile.bed", package='RCAS')
importBed(filePath = input, keepStandardChr = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_functions.R
\name{queryGff}
\alias{queryGff}
\title{queryGff}
\usage{
queryGff(queryRegions, gffData)
}
\arguments{
\item{queryRegions}{GRanges object imported from a BED file using
\code{importBed} function}

\item{gffData}{GRanges object imported from a GTF file using \code{importGtf}
function}
}
\value{
a GRanges object (a subset of input gff) with an additional column
  'overlappingQuery' that contains the coordinates of query regions that
  overlap the target annotation features
}
\description{
This function checks overlaps between the regions in input query and in
reference. Input query should be in BED format and reference should be in GFF
format. Both data are imported as GRanges object.
}
\examples{
data(queryRegions)
data(gff)
overlaps <- queryGff(queryRegions = queryRegions, gffData = gff)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_analysis.R
\name{getPWM}
\alias{getPWM}
\title{getPWM}
\usage{
getPWM(sequences, letters = c("A", "C", "G", "T"))
}
\arguments{
\item{sequences}{vector of strings of equal widths.}

\item{letters}{vector of characters to consider as the an alphabet}
}
\value{
A matrix of position-specific-weights.
}
\description{
Given a vector of strings of equal width, generate a position-specific 
weight matrix based on the frequency of occurrence of the unique letters
found in the sequences.
}
\examples{

sequences = c("GGAGAG", "GAAGAA", "TGAGAA", "GGAGAA", "GAAGAA")
getPWM(sequences)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_analysis.R
\name{getMotifSummaryTable}
\alias{getMotifSummaryTable}
\title{getMotifSummaryTable}
\usage{
getMotifSummaryTable(motifResults)
}
\arguments{
\item{motifResults}{Output object of \code{runMotifDiscovery} function}
}
\value{
A data.frame object containing summary statistics about the
  discovered motifs
}
\description{
Get summary stats for top discovered motifs
}
\examples{

data(queryRegions)
motifResults <- runMotifDiscovery(queryRegions = queryRegions[1:1000],
                                  genomeVersion = 'hg19',
                                  resize = 15,
                                  motifN = 1,
                                  maxMismatch = 1,
                                  nCores = 2)
motifSummary <- getMotifSummaryTable(motifResults)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment_analysis.R
\name{retrieveOrthologs}
\alias{retrieveOrthologs}
\title{retrieveOrthologs}
\usage{
retrieveOrthologs()
}
\description{
This function is deprecated. For functional enrichment 
analysis, use findEnrichedFunctions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta_analysis_functions.R
\name{runReportMetaAnalysis}
\alias{runReportMetaAnalysis}
\title{runReportMetaAnalysis}
\usage{
runReportMetaAnalysis(
  dbPath = "RCAS.sqlite",
  sampleTablePath,
  outDir = getwd(),
  outFile = NULL,
  quiet = FALSE,
  selfContained = TRUE
)
}
\arguments{
\item{dbPath}{Path to the sqlite database generated by RCAS::createDB}

\item{sampleTablePath}{A tab-separated file with two columns (no rownames) 
header 1: sampleName, header 2: sampleGroup}

\item{outDir}{Path to the output directory. (default: current working 
directory)}

\item{outFile}{Name of the output HTML report (by default, the base name of
sampleTablePath value is used to create a name for the HTML report)}

\item{quiet}{boolean value (default: FALSE). If set to TRUE, progress bars 
and chunk labels will be suppressed while knitting the Rmd file.}

\item{selfContained}{boolean value (default: TRUE). By default, the generated
html file will be self-contained, which means that all figures and tables 
will be embedded in a single html file with no external dependencies (See 
rmarkdown::html_document)}
}
\value{
An html generated using rmarkdown/knitr/pandoc that contains 
  interactive figures, tables, and text that provide an overview of the 
  experiment
}
\description{
Generate a stand-alone HTML report with interactive figures and tables from a
pre-calculated RCAS database (using RCAS::createDB) to compare multiple 
samples.
}
\examples{
dbPath <- system.file("extdata", "hg19.RCASDB.sqlite",
package='RCAS') 

#Hint: use RCAS::summarizeDatabaseContent to see which samples have processed
#data in the database. 
summarizeDatabaseContent(dbPath = dbPath)

#Create a data table for samples and their groups sampleGroup field is used
#to group replicates of #the same sample into one group in visualizations. 
#Any arbitrary name can be used for sampleGroup field. However, entries in
#the sampleName field must be available in the queried database
sampleData <- data.frame('sampleName' = c('FUS', 'FMR1'), 
    'sampleGroup' = c('FUS', 'FMR1'), stringsAsFactors = FALSE) 
write.table(sampleData, 'sampleDataTable.tsv', sep = '\t', 
    quote =FALSE, row.names = FALSE)
#Use the generated database to run a report 
runReportMetaAnalysis(dbPath = 'hg19.RCASDB.sqlite', 
    sampleTablePath = 'sampleDataTable.tsv')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_functions.R
\name{getFeatureBoundaryCoverage}
\alias{getFeatureBoundaryCoverage}
\title{getFeatureBoundaryCoverage}
\usage{
getFeatureBoundaryCoverage(
  queryRegions,
  featureCoords,
  flankSize = 500,
  boundaryType,
  sampleN = 0
)
}
\arguments{
\item{queryRegions}{GRanges object imported from a BED file using 
\code{importBed} function}

\item{featureCoords}{GRanges object containing the target feature coordinates}

\item{flankSize}{Positive integer that determines the number of base pairs to
extract around a given genomic feature boundary}

\item{boundaryType}{(Options: fiveprime or threeprime). Denotes which side of
the feature's boundary is to be profiled.}

\item{sampleN}{A positive integer value less than the total number of featuer
coordinates that determines whether the target feature coordinates should 
be randomly downsampled. If set to 0, no downsampling will happen. If}
}
\value{
a data frame containin three columns. 1. fivePrime: Coverage at 5' 
  end of features 2. threePrime: Coverage at 3' end of features; 3. bases: 
  distance (in bp) to the boundary
}
\description{
This function extracts the flanking regions of 5' and 3' boundaries of a 
given set of genomic features and computes the per-base coverage of query 
regions across these boundaries.
}
\examples{
data(queryRegions)
data(gff)
txdb <- txdbmaker::makeTxDbFromGRanges(gff)
transcriptCoords <- GenomicFeatures::transcripts(txdb)
transcriptEndCoverage <- getFeatureBoundaryCoverage (
                                     queryRegions = queryRegions,
                                    featureCoords = transcriptCoords,
                                    flankSize = 100,
                                    boundaryType = 'threeprime',
                                    sampleN = 1000)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_functions.R
\name{importGtf}
\alias{importGtf}
\title{importGtf}
\usage{
importGtf(
  filePath,
  saveObjectAsRds = TRUE,
  readFromRds = TRUE,
  overwriteObjectAsRds = FALSE,
  keepStandardChr = TRUE,
  ...
)
}
\arguments{
\item{filePath}{Path to a GTF file}

\item{saveObjectAsRds}{TRUE/FALSE (default:TRUE). If it is set to TRUE, a
GRanges object will be created and saved in RDS format
(<filePath>.granges.rds) so that importing can re-use this .rds file in
next run.}

\item{readFromRds}{TRUE/FALSE (default:TRUE). If it is set to TRUE,
annotation data will be imported from previously generated .rds file
(<filePath>.granges.rds).}

\item{overwriteObjectAsRds}{TRUE/FALSE (default:FALSE). If it is set to TRUE,
existing .rds file (<filePath>.granges.rds) will overwritten.}

\item{keepStandardChr}{TRUE/FALSE (default:TRUE). If it is set to TRUE,
\code{seqlevelsStyle} will be converted to 'UCSC' and
\code{keepStandardChromosomes} function  will be applied to only keep data
from the standard chromosomes.}

\item{...}{Other arguments passed to rtracklayer::import.gff function}
}
\value{
A \code{GRanges} object containing the coordinates of the annotated
  genomic features in an input GTF file
}
\description{
This function uses \code{rtracklayer::import.gff()} function to import genome
annoatation data from an Ensembl gtf file
}
\examples{
#import the data and write it into a .rds file
\dontrun{
importGtf(filePath='./Ensembl75.hg19.gtf')
}
#import the data but don't save it as RDS
\dontrun{
importGtf(filePath='./Ensembl75.hg19.gtf', saveObjectAsRds = FALSE)
}
#import the data and overwrite the previously generated
\dontrun{
importGtf(filePath='./Ensembl75.hg19.gtf', overwriteObjectAsRds = TRUE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/databaseSetUpFunctions.R
\name{deleteSampleDataFromDB}
\alias{deleteSampleDataFromDB}
\title{deleteSampleDataFromDB}
\usage{
deleteSampleDataFromDB(dbPath, sampleNames)
}
\arguments{
\item{dbPath}{Path to the sqlite database}

\item{sampleNames}{The names of the samples for which all relevant datasets
should be deleted from the database. Tip: Use RSQLite::dbReadTable function
to read the table 'processedSamples' to see which samples are available in
the database.}
}
\value{
SQLiteConnection object with updated contents in the dbPath
}
\description{
Given a list of sample names, the function deletes all datasets calculated
for the given samples from the database.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/databaseSetUpFunctions.R
\name{summarizeDatabaseContent}
\alias{summarizeDatabaseContent}
\title{summarizeDatabaseContent}
\usage{
summarizeDatabaseContent(dbPath)
}
\arguments{
\item{dbPath}{Path to the sqlite database}
}
\value{
A data.frame object
}
\description{
Given a path to an sqlite database created using RCAS::createDB function,
accesses the database and provides a quick summary of available samples 
and number of entries of each sample in the available tables of the 
database.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_analysis.R
\name{runMotifDiscovery}
\alias{runMotifDiscovery}
\title{runMotifDiscovery}
\usage{
runMotifDiscovery(
  queryRegions,
  resizeN = 0,
  motifWidth = 6,
  sampleN = 0,
  genomeVersion,
  maxMismatch = 1,
  motifN = 5,
  nCores = 1
)
}
\arguments{
\item{queryRegions}{GRanges object containing coordinates of input query
regions imported by the \code{\link{importBed}} function}

\item{resizeN}{Integer value (default: 0) to resize query regions if they are
shorter than the value of \code{resize}. Set to 0 to disable resize.}

\item{motifWidth}{A Positive integer (default: 6) for the generated k-mers.
Warning: we recommend using values below 10 as the computation gets
exponentially difficult as the motif width is increased.}

\item{sampleN}{A positive integer value. The queryRegions are randomly
downsampled to include intervals as many as \code{sampleN}. The input will
be downsampled only if this value is larger than zero and less than the
total number of input intervals.}

\item{genomeVersion}{A character string to denote the BS genome library
required to extract sequences. Example: 'hg19'}

\item{maxMismatch}{A positive integer (default: 1) - maximum number of
mismatches to allow when searching for k-mer matches in sequences.}

\item{motifN}{A positive integer (default:5) denoting the maximum number of
motifs that should be returned by the \code{findDifferentialMotifs}
function}

\item{nCores}{A positive integer (default:1) number of cores used for
parallel execution.}
}
\value{
A list of four objects: k-mer count matrices for query and background
  and lists of string matches for the top discriminating motifs (motifN).
}
\description{
This function builds a random forest classifier to find the top most
discriminative motifs in the query regions compared to the background. The
background sequences are automatically generated based on the query regions.
First, k-mers of a fixed length are generated. The query and control sequences
are searched for k-mers allowing for mismatches. A random forest model is 
trained to find the most discriminative motifs.
}
\examples{
data(queryRegions)
motifResults <- runMotifDiscovery(queryRegions = queryRegions[1:1000],
                                  genomeVersion = 'hg19',
                                  motifWidth = 6, 
                                  resize = 15,
                                  motifN = 1,
                                  maxMismatch = 1,
                                  nCores = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_functions.R
\name{calculateCoverageProfileList}
\alias{calculateCoverageProfileList}
\title{calculateCoverageProfileList}
\usage{
calculateCoverageProfileList(
  queryRegions,
  targetRegionsList,
  sampleN = 0,
  bin.num = 100,
  bin.op = "mean",
  strand.aware = TRUE
)
}
\arguments{
\item{queryRegions}{GRanges object imported from a BED file using 
\code{importBed} function}

\item{targetRegionsList}{A list of GRanges objects containing genomic 
coordinates of target features (e.g. transcripts, exons, introns)}

\item{sampleN}{If set to a positive integer, \code{targetRegions} will be 
downsampled to \code{sampleN} regions}

\item{bin.num}{Positive integer value (default: 100) to determine how many
bins the targetRegions should be split into (See
genomation::ScoreMatrixBin)}

\item{bin.op}{The operation to apply for each bin: 'min', 'max', or 'mean'
(default: mean). (See genomation::ScoreMatrixBin)}

\item{strand.aware}{TRUE/FALSE (default: TRUE) The strands of target regions
are considered.}
}
\value{
A data.frame consisting of four columns: 1. bins level 2.
  meanCoverage 3. standardError 4. feature Target regions are divided into
  100 equal sized bins and coverage level is summarized in a strand-specific
  manner using the \code{genomation::ScoreMatrixBin} function. For each bin,
  mean coverage score and the standard error of the mean coverage score is
  calculated (\code{plotrix::std.error})
}
\description{
This function checks overlaps between input query regions and a target list 
of annotation features, and then calculates the coverage profile along the 
target regions.
}
\examples{
data(gff)
data(queryRegions)
txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)
dfList <- calculateCoverageProfileList(queryRegions = queryRegions,
                              targetRegionsList = txdbFeatures,
                                    sampleN = 1000)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment_analysis.R
\name{runGSEA}
\alias{runGSEA}
\title{runGSEA}
\usage{
runGSEA()
}
\description{
This function is deprecated. For functional enrichment 
analysis, use findEnrichedFunctions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment_analysis.R
\name{printMsigdbDataset}
\alias{printMsigdbDataset}
\title{Print MSIGDB Dataset to a file}
\usage{
printMsigdbDataset()
}
\description{
This function is deprecated. For functional enrichment 
analysis, use findEnrichedFunctions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/motif_analysis.R
\name{runMotifRG}
\alias{runMotifRG}
\title{run motifRG}
\usage{
runMotifRG()
}
\description{
run motifRG
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_functions.R
\name{getTxdbFeatures}
\alias{getTxdbFeatures}
\title{getTxdbFeatures}
\usage{
getTxdbFeatures()
}
\description{
This function is deprecated. Use getTxdbFeaturesFromGRanges instead.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment_analysis.R
\name{findEnrichedFunctions}
\alias{findEnrichedFunctions}
\title{findEnrichedFunctions}
\usage{
findEnrichedFunctions(targetGenes, species, ...)
}
\arguments{
\item{targetGenes}{Vector of Ensembl gene ids or gene names}

\item{species}{First letter of genus + species name: e.g. hsapiens}

\item{...}{Other arguments to be passed to gprofiler2::gost}
}
\description{
Find enriched functional terms among the genes that overlap
the regions of interest.
}
\details{
This function is basically a call to gprofiler2::gost function. 
It is here to serve as a replacement for other deprecated functional
enrichment functions.
}
\examples{
data(gff)
data(queryRegions)

overlaps <- queryGff(queryRegions, gff)
res <- findEnrichedFunctions(unique(overlaps$gene_id), 'hsapiens')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gff}
\alias{gff}
\title{Sample GFF file imported as a GRanges object}
\format{
GRanges object with 238010 ranges and 16 metadata columns
}
\source{
\url{ftp://ftp.ensembl.org/pub/release-75/gtf/homo_sapiens/Homo_sapiens.GRCh37.75.gtf.gz}
}
\usage{
gff
}
\value{
A GRanges object
}
\description{
This dataset contains genomic annotation data from Ensembl version 75 for
Homo sapiens downloaded from Ensembl. The GFF file is imported via the
\code{importGtf} function and a subset of the data is selected by choosing
features found on 'chr1'.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta_analysis_functions.R
\name{summarizeQueryRegionsMulti}
\alias{summarizeQueryRegionsMulti}
\title{summarizeQueryRegionsMulti}
\usage{
summarizeQueryRegionsMulti(queryRegionsList, txdbFeatures, nodeN = 1)
}
\arguments{
\item{queryRegionsList}{GRangesList object imported from multiple BED files
using \code{importBedFiles} function}

\item{txdbFeatures}{List of GRanges objects - outputs of 
\code{getTxdbFeaturesFromGRanges} and \code{getTxdbFeatures} functions}

\item{nodeN}{Positive integer value that denotes the number of cpus to use
for parallel processing (default: 1)}
}
\value{
A list consisting of two data.frame objects: one for raw overlap 
  counts and one for percentage of overlap counts (raw overlap counts divided
  by the number of query regions in the corresponding BED file)
}
\description{
This function is a wrapper function to run RCAS::summarizeQueryRegions 
multiple times, which is useful to get a matrix of overlap counts between a 
list of BED files with a txdbFeatures extracted from GTF file
}
\examples{
data(gff)
data(queryRegions)
queryRegionsList <- GenomicRanges::GRangesList(queryRegions, queryRegions)
names(queryRegionsList) <- c('q1', 'q2')
txdbFeatures <- getTxdbFeaturesFromGRanges(gffData = gff)
summaryMatrix <- summarizeQueryRegionsMulti(queryRegionsList = queryRegionsList,
                                 txdbFeatures = txdbFeatures)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{queryRegions}
\alias{queryRegions}
\title{Sample BED file imported as a GRanges object}
\format{
GRanges object with 10000 ranges and 2 metadata columns
}
\source{
\url{http://dorina.mdc-berlin.de/regulators}
}
\usage{
queryRegions
}
\value{
A GRanges object
}
\description{
This dataset contains a randomly selected sample of human LIN28A protein
binding sites detected by HITS-CLIP analysis downloaded from DoRina database
(LIN28A HITS-CLIP hESCs (Wilbert 2012)). The BED file is imported via the
\code{importBed} function and a subset of the data is selected by randomly
choosing 10000 regions.
}
\keyword{datasets}
